@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -61,7 +61,7 @@ namespace internal {
     class function_input_base : public receiver<Input>, tbb::internal::no_assign {
         typedef sender<Input> predecessor_type;
         enum op_stat {WAIT=0, SUCCEEDED, FAILED};
-        enum op_type {reg_pred, rem_pred, app_body, tryput, try_fwd};
+        enum op_type {reg_pred, rem_pred, app_body, try_fwd, tryput_bypass, app_body_bypass };
         typedef function_input_base<Input, A, ImplType> my_class;
         
     public:
@@ -92,18 +92,20 @@ namespace internal {
             if ( my_queue ) delete my_queue;
         }
         
-        //! Put to the node
-        virtual bool try_put( const input_type &t ) {
+        //! Put to the node, returning a task if available
+        virtual task * try_put_task( const input_type &t ) {
            if ( my_max_concurrency == 0 ) {
-               spawn_body_task( t );
-               return true;
+               return create_body_task( t );
            } else {
-               my_operation op_data(t, tryput);
+               my_operation op_data(t, tryput_bypass);
                my_aggregator.execute(&op_data);
-               return op_data.status == SUCCEEDED;
+               if(op_data.status == SUCCEEDED ) {
+                   return op_data.bypass_t;
+               }
+               return NULL;
            }
         }
-        
+
         //! Adds src to the list of cached predecessors.
         /* override */ bool register_predecessor( predecessor_type &src ) {
             my_operation op_data(reg_pred);
@@ -122,16 +124,29 @@ namespace internal {
 
     protected:
 
+        void reset_function_input_base() {
+            my_concurrency = 0;
+            if(my_queue) {
+                my_queue->reset();
+            }
+            my_predecessors.reset();
+            forwarder_busy = false;
+        }
+
         task *my_root_task;
         const size_t my_max_concurrency;
         size_t my_concurrency;
         function_input_queue<input_type, A> *my_queue;
         predecessor_cache<input_type, null_mutex > my_predecessors;
         
+        /*override*/void reset_receiver() {
+            my_predecessors.reset();
+        }
+
     private:
 
-        friend class apply_body_task< my_class, input_type >;
-        friend class forward_task< my_class >;
+        friend class apply_body_task_bypass< my_class, input_type >;
+        friend class forward_task_bypass< my_class >;
         
         class my_operation : public aggregated_operation< my_operation > {
         public:
@@ -140,6 +155,7 @@ namespace internal {
                 input_type *elem;
                 predecessor_type *r;
             };
+            tbb::task *bypass_t;
             my_operation(const input_type& e, op_type t) :
                 type(char(t)), elem(const_cast<input_type*>(&e)) {}
             my_operation(op_type t) : type(char(t)), r(NULL) {}
@@ -185,28 +201,52 @@ namespace internal {
                         }
                     }
                     break;
-                case tryput: internal_try_put(tmp);  break;
+                case app_body_bypass: {
+                        task * new_task = NULL;
+                        __TBB_ASSERT(my_max_concurrency != 0, NULL);
+                        --my_concurrency;
+                        if (my_concurrency<my_max_concurrency) {
+                            input_type i;
+                            bool item_was_retrieved = false;
+                            if ( my_queue )
+                                item_was_retrieved = my_queue->pop(i);
+                            else 
+                                item_was_retrieved = my_predecessors.get_item(i);
+                            if (item_was_retrieved) {
+                                ++my_concurrency;
+                                new_task = create_body_task(i);
+                            }
+                        }
+                        tmp->bypass_t = new_task;
+                        __TBB_store_with_release(tmp->status, SUCCEEDED);
+                    }
+                    break;
+                case tryput_bypass: internal_try_put_task(tmp);  break;
                 case try_fwd: internal_forward(tmp);  break;
-                }
+                    }
             }
         }
         
-        //! Put to the node
-        void internal_try_put(my_operation *op) {
+        //! Put to the node, but return the task instead of enqueueing it
+        void internal_try_put_task(my_operation *op) {
             __TBB_ASSERT(my_max_concurrency != 0, NULL);
             if (my_concurrency < my_max_concurrency) {
                ++my_concurrency;
-               spawn_body_task(*(op->elem));
+               task * new_task = create_body_task(*(op->elem));
+               op->bypass_t = new_task;
                __TBB_store_with_release(op->status, SUCCEEDED);
            } else if ( my_queue && my_queue->push(*(op->elem)) ) { 
+               op->bypass_t = SUCCESSFULLY_ENQUEUED;
                __TBB_store_with_release(op->status, SUCCEEDED);
            } else {
+               op->bypass_t = NULL;
                __TBB_store_with_release(op->status, FAILED);
            }
         }
         
         //! Tries to spawn bodies if available and if concurrency allows
         void internal_forward(my_operation *op) {
+            op->bypass_t = NULL;
             if (my_concurrency<my_max_concurrency || !my_max_concurrency) {
                 input_type i;
                 bool item_was_retrieved = false;
@@ -216,8 +256,8 @@ namespace internal {
                     item_was_retrieved = my_predecessors.get_item(i);
                 if (item_was_retrieved) {
                     ++my_concurrency;
+                    op->bypass_t = create_body_task(i);
                     __TBB_store_with_release(op->status, SUCCEEDED);
-                    spawn_body_task(i);
                     return;
                 }
             }
@@ -226,31 +266,62 @@ namespace internal {
         }
         
         //! Applies the body to the provided input
+        //  then decides if more work is available 
         void apply_body( input_type &i ) {
-            static_cast<ImplType *>(this)->apply_body_impl(i);
-            if ( my_max_concurrency != 0 ) {
-                my_operation op_data(app_body);
-                my_aggregator.execute(&op_data);
-            }
+            task *new_task = apply_body_bypass(i);
+            if(!new_task) return;
+            if(new_task == SUCCESSFULLY_ENQUEUED) return;
+            task::enqueue(*new_task);
+            return;
         }
         
+        //! Applies the body to the provided input
+        //  then decides if more work is available 
+        task * apply_body_bypass( input_type &i ) {
+            task * new_task = static_cast<ImplType *>(this)->apply_body_impl_bypass(i);
+            if ( my_max_concurrency != 0 ) {
+                my_operation op_data(app_body_bypass);  // tries to pop an item or get_item, enqueues another apply_body
+                my_aggregator.execute(&op_data);
+                tbb::task *ttask = op_data.bypass_t;
+                new_task = combine_tasks(new_task, ttask);
+            }
+            return new_task;
+        }
+        
+       //! allocates a task to call apply_body( input )
+       inline task * create_body_task( const input_type &input ) {
+           return new(task::allocate_additional_child_of(*my_root_task))
+               apply_body_task_bypass < my_class, input_type >(*this, input);
+       }
+
        //! Spawns a task that calls apply_body( input )
        inline void spawn_body_task( const input_type &input ) {
-           task::enqueue(*new(task::allocate_additional_child_of(*my_root_task)) apply_body_task< my_class, input_type >(*this, input));
+           task::enqueue(*create_body_task(input));
        }
         
        //! This is executed by an enqueued task, the "forwarder"
-       void forward() {
+       task *forward_task() {
            my_operation op_data(try_fwd);
+           task *rval = NULL;
            do {
                op_data.status = WAIT;
                my_aggregator.execute(&op_data);
+               if(op_data.status == SUCCEEDED) {
+                   tbb::task *ttask = op_data.bypass_t;
+                   rval = combine_tasks(rval, ttask);
+               }
            } while (op_data.status == SUCCEEDED);
+           return rval;
        }
         
+       inline task *create_forward_task() {
+           task *rval = new(task::allocate_additional_child_of(*my_root_task)) forward_task_bypass< my_class >(*this);
+           return rval;
+       }
+
        //! Spawns a task that calls forward()
        inline void spawn_forward_task() {
-           task::enqueue(*new(task::allocate_additional_child_of(*my_root_task)) forward_task< my_class >(*this));
+           task::enqueue(*create_forward_task());
        }
     };  // function_input_base
 
@@ -289,72 +360,86 @@ namespace internal {
             return dynamic_cast< internal::function_body_leaf<input_type, output_type, Body> & >(body_ref).get_body(); 
         } 
 
-        void apply_body_impl( const input_type &i) {
-            successors().try_put( (*my_body)(i) );
+        task * apply_body_impl_bypass( const input_type &i) {
+            task * new_task = successors().try_put_task( (*my_body)(i) );
+            return new_task;
         }
 
     protected:
+
+        void reset_function_input() { 
+            base_type::reset_function_input_base();
+        }
+
         function_body<input_type, output_type> *my_body;
         virtual broadcast_cache<output_type > &successors() = 0;
 
     };
 
-#if TBB_PREVIEW_GRAPH_NODES
     //! Implements methods for a function node that takes a type Input as input
     //  and has a tuple of output ports specified.  
     template< typename Input, typename OutputPortSet, typename A>
-    class multioutput_function_input : public function_input_base<Input, A, multioutput_function_input<Input,OutputPortSet,A> > {
+    class multifunction_input : public function_input_base<Input, A, multifunction_input<Input,OutputPortSet,A> > {
     public:
         typedef Input input_type;
         typedef OutputPortSet output_ports_type;
-        typedef multioutput_function_input<Input,OutputPortSet,A> my_class;
+        typedef multifunction_input<Input,OutputPortSet,A> my_class;
         typedef function_input_base<Input, A, my_class> base_type;
         typedef function_input_queue<input_type, A> input_queue_type;
 
 
         // constructor
         template<typename Body>
-        multioutput_function_input( 
+        multifunction_input( 
                 graph &g, 
                 size_t max_concurrency, 
                 Body& body,
                 function_input_queue<input_type,A> *q = NULL ) :
             base_type(g, max_concurrency, q),
-            my_body( new internal::multioutput_function_body_leaf<input_type, output_ports_type, Body>(body) ) {
+            my_body( new internal::multifunction_body_leaf<input_type, output_ports_type, Body>(body) ) {
         }
 
         //! Copy constructor
-        multioutput_function_input( const multioutput_function_input& src, input_queue_type *q = NULL ) : 
+        multifunction_input( const multifunction_input& src, input_queue_type *q = NULL ) : 
                 base_type(src, q),
                 my_body( src.my_body->clone() ) {
         }
 
-        ~multioutput_function_input() {
+        ~multifunction_input() {
             delete my_body;
         }
 
         template< typename Body >
         Body copy_function_object() {
-            internal::multioutput_function_body<input_type, output_ports_type> &body_ref = *this->my_body;
-            return dynamic_cast< internal::multioutput_function_body_leaf<input_type, output_ports_type, Body> & >(body_ref).get_body(); 
+            internal::multifunction_body<input_type, output_ports_type> &body_ref = *this->my_body;
+            return dynamic_cast< internal::multifunction_body_leaf<input_type, output_ports_type, Body> & >(body_ref).get_body(); 
         } 
 
-        void apply_body_impl( const input_type &i) {
+        // for multifunction nodes we do not have a single successor as such.  So we just tell
+        // the task we were successful.
+        task * apply_body_impl_bypass( const input_type &i) {
             (*my_body)(i, my_output_ports);
+            task * new_task = SUCCESSFULLY_ENQUEUED;
+            return new_task;
         }
 
         output_ports_type &output_ports(){ return my_output_ports; }
 
     protected:
-        multioutput_function_body<input_type, output_ports_type> *my_body;
+
+        void reset() {
+            base_type::reset_function_input_base();
+        }
+
+        multifunction_body<input_type, output_ports_type> *my_body;
         output_ports_type my_output_ports;
 
     };
 
-    // template to refer to an output port of a multioutput_function_node
+    // template to refer to an output port of a multifunction_node
     template<size_t N, typename MOP>
-    typename std::tuple_element<N, typename MOP::output_ports_type>::type &output_port(MOP &op) {
-        return std::get<N>(op.output_ports()); 
+    typename tbb::flow::tuple_element<N, typename MOP::output_ports_type>::type &output_port(MOP &op) {
+        return tbb::flow::get<N>(op.output_ports()); 
     }
 
 // helper structs for split_node
@@ -362,7 +447,7 @@ namespace internal {
     struct emit_element {
         template<typename T, typename P>
         static void emit_this(const T &t, P &p) {
-            std::get<N-1>(p).put(std::get<N-1>(t));
+            (void)tbb::flow::get<N-1>(p).try_put(tbb::flow::get<N-1>(t));
             emit_element<N-1>::emit_this(t,p);
         }
     };
@@ -371,10 +456,9 @@ namespace internal {
     struct emit_element<1> {
         template<typename T, typename P>
         static void emit_this(const T &t, P &p) {
-            std::get<0>(p).put(std::get<0>(t));
+            (void)tbb::flow::get<0>(p).try_put(tbb::flow::get<0>(t));
         }
     };
-#endif  // TBB_PREVIEW_GRAPH_NODES
 
     //! Implements methods for an executable node that takes continue_msg as input
     template< typename Output >
@@ -400,6 +484,10 @@ namespace internal {
         continue_input( const continue_input& src ) : continue_receiver(src), 
             my_root_task(src.my_root_task), my_body( src.my_body->clone() ) {}
 
+        ~continue_input() {
+            delete my_body;
+        }
+
         template< typename Body >
         Body copy_function_object() {
             internal::function_body<input_type, output_type> &body_ref = *my_body;
@@ -413,17 +501,18 @@ namespace internal {
         
         virtual broadcast_cache<output_type > &successors() = 0; 
         
-        friend class apply_body_task< continue_input< Output >, continue_msg >;
+        friend class apply_body_task_bypass< continue_input< Output >, continue_msg >;
         
         //! Applies the body to the provided input
-        /* override */ void apply_body( input_type ) {
-            successors().try_put( (*my_body)( continue_msg() ) );
+        /* override */ task *apply_body_bypass( input_type ) {
+            return successors().try_put_task( (*my_body)( continue_msg() ) );
         }
         
         //! Spawns a task that applies the body
-        /* override */ void execute( ) {
-            task::enqueue( * new ( task::allocate_additional_child_of( *my_root_task ) ) 
-               apply_body_task< continue_input< Output >, continue_msg >( *this, continue_msg() ) ); 
+        /* override */ task *execute( ) {
+            task *res = new ( task::allocate_additional_child_of( *my_root_task ) ) 
+                apply_body_task_bypass< continue_input< Output >, continue_msg >( *this, continue_msg() ); 
+            return res;
         }
 
     };
@@ -452,15 +541,14 @@ namespace internal {
             return true;
         }
 
-        // for multioutput_function_node.  The function_body that implements
+        // for multifunction_node.  The function_body that implements
         // the node will have an input and an output tuple of ports.  To put
         // an item to a successor, the body should
         //
-        //    get<I>(output_ports).put(output_value);
+        //    get<I>(output_ports).try_put(output_value);
         //
-        // successors must always accept (for instance, a queue_node) or items
-        // may be dropped.
-        void put(const output_type &i) { my_successors.try_put(i); }
+        // return value will be bool returned from successors.try_put.
+        task *try_put_task(const output_type &i) { return my_successors.try_put_task(i); }
           
     protected:
         broadcast_cache<output_type> my_successors;
@@ -468,6 +556,24 @@ namespace internal {
         
     };
 
+    template< typename Output >
+    class multifunction_output : public function_output<Output> {
+    public:
+        typedef Output output_type;
+        typedef function_output<output_type> base_type;
+        using base_type::my_successors;
+        
+        multifunction_output() : base_type() {my_successors.set_owner(this);}
+        multifunction_output( const multifunction_output &/*other*/) : base_type() { my_successors.set_owner(this); }
+
+        bool try_put(const output_type &i) {
+            task *res = my_successors.try_put_task(i);
+            if(!res) return false;
+            if(res != SUCCESSFULLY_ENQUEUED) task::enqueue(*res);
+            return true;
+        }
+    };
+
 }  // internal
 
 #endif // __TBB__flow_graph_node_impl_H
