@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -31,8 +31,10 @@
 
 // Support for dynamic loading entry points from other shared libraries.
 
+#include "tbb/tbb_stddef.h"
+
 #ifndef LIBRARY_ASSERT
-    #include "tbb/tbb_config.h"
+    #define LIBRARY_ASSERT(x,y) __TBB_ASSERT_EX(x,y)
 #endif /* !LIBRARY_ASSERT */
 
 /** By default, symbols declared and defined here go into namespace tbb::internal.
@@ -44,76 +46,72 @@
 #endif /* OPEN_INTERNAL_NAMESPACE */
 
 #include <stddef.h>
-#if _WIN32||_WIN64
+#if _WIN32
 #include "tbb/machine/windows_api.h"
-#endif /* _WIN32||_WIN64 */
+#endif /* _WIN32 */
 
 OPEN_INTERNAL_NAMESPACE
 
 //! Type definition for a pointer to a void somefunc(void)
 typedef void (*pointer_to_handler)();
 
-// Double cast through the void* from func_ptr in DLD macro is necessary to
+//! The helper to construct dynamic_link_descriptor structure
+// Double cast through the void* in DLD macro is necessary to
 // prevent warnings from some compilers (g++ 4.1)
-#if __TBB_WEAK_SYMBOLS
-
-#define DLD(s,h) {(pointer_to_handler)&s, (pointer_to_handler*)(void*)(&h)}
-//! Association between a handler name and location of pointer to it.
-struct dynamic_link_descriptor {
-    //! pointer to the handler
-    pointer_to_handler ptr;
-    //! Pointer to the handler
-    pointer_to_handler* handler;
-};
-
-#else /* !__TBB_WEAK_SYMBOLS */
-
+#if __TBB_WEAK_SYMBOLS_PRESENT
+#define DLD(s,h) {#s, (pointer_to_handler*)(void*)(&h), (pointer_to_handler)&s}
+#else 
 #define DLD(s,h) {#s, (pointer_to_handler*)(void*)(&h)}
+#endif /* __TBB_WEAK_SYMBOLS_PRESENT */
 //! Association between a handler name and location of pointer to it.
 struct dynamic_link_descriptor {
     //! Name of the handler
     const char* name;
     //! Pointer to the handler
     pointer_to_handler* handler;
+#if __TBB_WEAK_SYMBOLS_PRESENT
+    //! Weak symbol
+    pointer_to_handler ptr;
+#endif
 };
 
-#endif /* !__TBB_WEAK_SYMBOLS */
-
-#if _WIN32||_WIN64
+#if _WIN32
 typedef HMODULE dynamic_link_handle;
 #else
 typedef void* dynamic_link_handle;
-#endif /* _WIN32||_WIN64 */
+#endif /* _WIN32 */
+
+const int DYNAMIC_LINK_GLOBAL = 0x01;
+const int DYNAMIC_LINK_LOAD   = 0x02;
+const int DYNAMIC_LINK_WEAK   = 0x04;
+const int DYNAMIC_LINK_ALL    = DYNAMIC_LINK_GLOBAL | DYNAMIC_LINK_LOAD | DYNAMIC_LINK_WEAK;
 
 //! Fill in dynamically linked handlers.
-/** 'n' is the length of the array descriptors[].
-    'required' is the number of the initial entries in the array descriptors[]
+/** 'required' is the number of the initial entries in the array descriptors[]
     that have to be found in order for the call to succeed. If the library and
     all the required handlers are found, then the corresponding handler pointers
     are set, and the return value is true.  Otherwise the original array of
-    descriptors is left untouched and the return value is false. **/
-bool dynamic_link( const char* libraryname,
+    descriptors is left untouched and the return value is false. 'required' is
+    limited by 20 (exceeding of this value will result in failure to load the
+    symbols and the return value will be false).
+    'dl_allowed' flag allows dynamic library loading if the global symbols
+    searching mechanism has failed.
+**/
+bool dynamic_link( const char* library,
                    const dynamic_link_descriptor descriptors[],
-                   size_t n,
-                   size_t required = ~(size_t)0,
-                   dynamic_link_handle* handle = 0 );
-
-bool dynamic_link( dynamic_link_handle module,
-                   const dynamic_link_descriptor descriptors[],
-                   size_t n,
-                   size_t required  = ~(size_t)0 );
+                   size_t required,
+                   dynamic_link_handle* handle = 0,
+                   int flags = DYNAMIC_LINK_ALL );
 
 void dynamic_unlink( dynamic_link_handle handle );
 
-#if __TBB_BUILD
 void dynamic_unlink_all();
-#endif /* __TBB_BUILD */
 
 enum dynamic_link_error_t {
     dl_success = 0,
     dl_lib_not_found,     // char const * lib, dlerr_t err
     dl_sym_not_found,     // char const * sym, dlerr_t err
-                          // Note: dlerr_t depends on OS: it is char const * on Linux and Mac, int on Windows.
+                          // Note: dlerr_t depends on OS: it is char const * on Linux* and OS X*, int on Windows*.
     dl_sys_fail,          // char const * func, int err
     dl_buff_too_small     // none
 }; // dynamic_link_error_t
