@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -29,15 +29,6 @@
 #ifndef _TBB_malloc_Customize_H_
 #define _TBB_malloc_Customize_H_
 
-/* Thread shutdown notification callback */
-/* redefine the name of the callback to meet TBB requirements
-   for externally visible names of service functions */
-#define mallocThreadShutdownNotification __TBB_mallocThreadShutdownNotification
-#define mallocProcessShutdownNotification __TBB_mallocProcessShutdownNotification
-
-extern "C" void mallocThreadShutdownNotification(void *);
-extern "C" void mallocProcessShutdownNotification(void);
-
 // customizing MALLOC_ASSERT macro
 #include "tbb/tbb_stddef.h"
 #define MALLOC_ASSERT(assertion, message) __TBB_ASSERT(assertion, message)
@@ -71,29 +62,25 @@ extern "C" void mallocProcessShutdownNotification(void);
     in a strict block-scoped locking pattern.  Omitting these methods permitted
     further simplification. */
 class MallocMutex : tbb::internal::no_copy {
-    __TBB_atomic_flag value;
+    __TBB_atomic_flag flag;
 
 public:
     class scoped_lock : tbb::internal::no_copy {
-        __TBB_Flag unlock_value;
         MallocMutex& mutex;
+        bool taken;
     public:
-        scoped_lock( MallocMutex& m ) : unlock_value(__TBB_LockByte(m.value)), mutex(m) {}
-        scoped_lock( MallocMutex& m, bool block, bool *locked ) : mutex(m) {
-            unlock_value = 1;
+        scoped_lock( MallocMutex& m ) : mutex(m), taken(true) { __TBB_LockByte(m.flag); }
+        scoped_lock( MallocMutex& m, bool block, bool *locked ) : mutex(m), taken(false) {
             if (block) {
-                unlock_value = __TBB_LockByte(m.value);
-                if (locked) *locked = true;
+                __TBB_LockByte(m.flag);
+                taken = true;
             } else {
-                if (bool res = __TBB_TryLockByte(m.value)) {
-                    unlock_value = 0;
-                    if (locked) *locked = true;
-                } else
-                    if (locked) *locked = false;
+                taken = __TBB_TryLockByte(m.flag);
             }
+            if (locked) *locked = taken;
         }
         ~scoped_lock() {
-            if (!unlock_value) __TBB_UnlockByte(mutex.value, unlock_value);
+            if (taken) __TBB_UnlockByte(mutex.flag);
         }
     };
     friend class scoped_lock;
@@ -123,10 +110,26 @@ inline void SpinWaitWhileEq(const volatile intptr_t &location, const intptr_t va
     tbb::internal::spin_wait_while_eq(location, value);
 }
 
+inline void SpinWaitUntilEq(const volatile intptr_t &location, const intptr_t value) {
+    tbb::internal::spin_wait_until_eq(location, value);
+}
+
 inline intptr_t BitScanRev(uintptr_t x) {
     return !x? -1 : __TBB_Log2(x);
 }
 
+template<typename T>
+static inline bool isAligned(T* arg, uintptr_t alignment) {
+    return tbb::internal::is_aligned(arg,alignment);
+}
+
+static inline bool isPowerOfTwo(uintptr_t arg) {
+    return tbb::internal::is_power_of_two(arg);
+}
+static inline bool isPowerOfTwoMultiple(uintptr_t arg, uintptr_t divisor) {
+    return arg && tbb::internal::is_power_of_two_factor(arg,divisor);
+}
+
 inline void AtomicOr(volatile void *operand, uintptr_t addend) {
     __TBB_AtomicOR(operand, addend);
 }
@@ -140,7 +143,7 @@ inline void AtomicAnd(volatile void *operand, uintptr_t addend) {
 // To support malloc replacement with LD_PRELOAD
 #include "proxy.h"
 
-#if MALLOC_LD_PRELOAD
+#if MALLOC_UNIXLIKE_OVERLOAD_ENABLED
 #define malloc_proxy __TBB_malloc_proxy
 extern "C" void * __TBB_malloc_proxy(size_t)  __attribute__ ((weak));
 #else
