@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -31,12 +31,14 @@
 
 // Define preprocessor symbols used to determine architecture
 #if _WIN32||_WIN64
-#   if defined(_M_AMD64)
+#   if defined(_M_X64)||defined(__x86_64__)  // the latter for MinGW support
 #       define __ARCH_x86_64 1
 #   elif defined(_M_IA64)
 #       define __ARCH_ipf 1
 #   elif defined(_M_IX86)||defined(__i386__) // the latter for MinGW support
 #       define __ARCH_x86_32 1
+#   elif defined(_M_ARM)
+#       define __ARCH_other 1
 #   else
 #       error Unknown processor architecture for Windows
 #   endif
@@ -54,13 +56,25 @@
 #   define USE_PTHREAD 1
 #endif
 
+// According to C99 standard INTPTR_MIN defined for C++
+// iff __STDC_LIMIT_MACROS pre-defined
+#ifndef __STDC_LIMIT_MACROS
+#define __STDC_LIMIT_MACROS 1
+#endif
+
+//! PROVIDE YOUR OWN Customize.h IF YOU FEEL NECESSARY
+#include "Customize.h"
+
 // Include files containing declarations of intptr_t and uintptr_t
 #include <stddef.h>  // size_t
 #if _MSC_VER
 typedef unsigned __int16 uint16_t;
 typedef unsigned __int32 uint32_t;
 typedef unsigned __int64 uint64_t;
-#else
+ #if !UINTPTR_MAX
+  #define UINTPTR_MAX SIZE_MAX
+ #endif
+#else // _MSC_VER
 #include <stdint.h>
 #endif
 
@@ -73,9 +87,6 @@ extern void  (*original_free_ptr)(void*);
 
 } } // namespaces
 
-//! PROVIDE YOUR OWN Customize.h IF YOU FEEL NECESSARY
-#include "Customize.h"
-
 /*
  * Functions to align an integer down or up to the given power of two,
  * and test for such an alignment, and for power of two.
@@ -89,17 +100,12 @@ static inline T alignUp  (T arg, uintptr_t alignment) {
     return T(((uintptr_t)arg+(alignment-1)) & ~(alignment-1));
     // /*is this better?*/ return (((uintptr_t)arg-1) | (alignment-1)) + 1;
 }
-template<typename T>
-static inline bool isAligned(T arg, uintptr_t alignment) {
-    return 0==((uintptr_t)arg & (alignment-1));
-}
-static inline bool isPowerOfTwo(uintptr_t arg) {
-    return arg && (0==(arg & (arg-1)));
-}
-static inline bool isPowerOfTwoMultiple(uintptr_t arg, uintptr_t divisor) {
-    // Divisor is assumed to be a power of two (which is valid for current uses).
-    MALLOC_ASSERT( isPowerOfTwo(divisor), "Divisor should be a power of two" );
-    return arg && (0==(arg & (arg-divisor)));
+template<typename T> // works for not power-of-2 alignments
+static inline T alignUpGeneric(T arg, uintptr_t alignment) {
+    if (size_t rem = arg % alignment) {
+        arg += alignment - rem;
+    }
+    return arg;
 }
 
 #endif /* _itt_shared_malloc_TypeDefinitions_H_ */
