@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -37,6 +37,7 @@
 #define tbb_tests_harness_H
 
 #include "tbb/tbb_config.h"
+#include "harness_defs.h"
 
 namespace Harness {
     enum TestResult {
@@ -61,17 +62,10 @@ namespace Harness {
     before including harness.h **/
 int TestMain ();
 
-#define __TBB_LAMBDAS_PRESENT  ( _MSC_VER >= 1600 && !__INTEL_COMPILER || __INTEL_COMPILER > 1100 && _TBB_CPP0X )
-
-#if defined(_MSC_VER) && _MSC_VER < 1400
-    #define __TBB_EXCEPTION_TYPE_INFO_BROKEN 1
-#else
-    #define __TBB_EXCEPTION_TYPE_INFO_BROKEN 0
-#endif
-
 #if __SUNPRO_CC
     #include <stdlib.h>
     #include <string.h>
+    #include <ucontext.h>
 #else /* !__SUNPRO_CC */
     #include <cstdlib>
 #if !TBB_USE_EXCEPTIONS && _MSC_VER
@@ -87,49 +81,129 @@ int TestMain ();
 
 #include <new>
 
+#if __TBB_MIC_NATIVE
+    #include "harness_mic.h"
+#else
     #define HARNESS_EXPORT
     #define REPORT_FATAL_ERROR REPORT
+#endif /* !__MIC__ */
 
 #if _WIN32||_WIN64
     #include "tbb/machine/windows_api.h"
-#if _XBOX
-    #undef HARNESS_NO_PARSE_COMMAND_LINE
-    #define HARNESS_NO_PARSE_COMMAND_LINE 1
-#endif
+    #if _WIN32_WINNT > 0x0501 && _MSC_VER
+        #include <dbghelp.h>
+        #pragma comment (lib, "dbghelp.lib")
+    #endif
+    #if _XBOX
+        #undef HARNESS_NO_PARSE_COMMAND_LINE
+        #define HARNESS_NO_PARSE_COMMAND_LINE 1
+    #endif
+    #if __TBB_WIN8UI_SUPPORT
+        #include <thread>
+    #endif
+    #if _MSC_VER
+        #include <crtdbg.h>
+    #endif
     #include <process.h>
 #else
     #include <pthread.h>
 #endif
+
 #if __linux__
     #include <sys/utsname.h> /* for uname */
     #include <errno.h>       /* for use in LinuxKernelVersion() */
+    #include <features.h>
+#endif
+// at least GLIBC 2.1 or OSX 10.5
+#if __GLIBC__>2 || ( __GLIBC__==2 && __GLIBC_MINOR__ >= 1) || __APPLE__
+    #include <execinfo.h> /*backtrace*/
+    #define BACKTRACE_FUNCTION_AVAILABLE 1
 #endif
 
 #include "harness_report.h"
 
+#if HARNESS_USE_RUNTIME_LOADER
+    #define TBB_PREVIEW_RUNTIME_LOADER 1
+    #include "tbb/runtime_loader.h"
+    static char const * _path[] = { ".", NULL };
+    static tbb::runtime_loader _runtime_loader( _path );
+#endif // HARNESS_USE_RUNTIME_LOADER
+
 #if !HARNESS_NO_ASSERT
+
 #include "harness_assert.h"
+#if TEST_USES_TBB
+#include <tbb/tbb_stddef.h> /*set_assertion_handler*/
+
+struct InitReporter {
+    InitReporter() {
+#if TBB_USE_ASSERT
+        tbb::set_assertion_handler(ReportError);
+#endif
+        ASSERT_WARNING(TBB_INTERFACE_VERSION <= tbb::TBB_runtime_interface_version(), "runtime version mismatch");
+    }
+};
+static InitReporter InitReportError;
+#endif
 
 typedef void (*test_error_extra_t)(void);
 static test_error_extra_t ErrorExtraCall;
 //! Set additional handler to process failed assertions
 void SetHarnessErrorProcessing( test_error_extra_t extra_call ) {
     ErrorExtraCall = extra_call;
-    // TODO: add tbb::set_assertion_handler(ReportError);
 }
+
 //! Reports errors issued by failed assertions
 void ReportError( const char* filename, int line, const char* expression, const char * message ) {
+#if BACKTRACE_FUNCTION_AVAILABLE
+    const int sz = 100; // max number of frames to capture
+    void *buff[sz];
+    int n = backtrace(buff, sz);
+    REPORT("Call stack info (%d):\n", n);
+    backtrace_symbols_fd(buff, n, fileno(stdout));
+#elif __SUNPRO_CC
+    REPORT("Call stack info:\n");
+    printstack(fileno(stdout));
+#elif _WIN32_WINNT > 0x0501 && _MSC_VER && !__TBB_WIN8UI_SUPPORT
+    const int sz = 62; // XP limitation for number of frames
+    void *buff[sz];
+    int n = CaptureStackBackTrace(0, sz, buff, NULL);
+    REPORT("Call stack info (%d):\n", n);
+    static LONG once = 0;
+    if( !InterlockedExchange(&once, 1) )
+        SymInitialize(GetCurrentProcess(), NULL, TRUE);
+    const int len = 255; // just some reasonable string buffer size
+    union { SYMBOL_INFO sym; char pad[sizeof(SYMBOL_INFO)+len]; };
+    sym.MaxNameLen = len;
+    sym.SizeOfStruct = sizeof( SYMBOL_INFO );
+    DWORD64 offset;
+    for(int i = 1; i < n; i++) { // skip current frame
+        if(!SymFromAddr( GetCurrentProcess(), DWORD64(buff[i]), &offset, &sym )) {
+            sym.Address = ULONG64(buff[i]); offset = 0; sym.Name[0] = 0;
+        }
+        REPORT("[%d] %016I64LX+%04I64LX: %s\n", i, sym.Address, offset, sym.Name); //TODO: print module name
+    }
+#endif /*BACKTRACE_FUNCTION_AVAILABLE*/
+
 #if __TBB_ICL_11_1_CODE_GEN_BROKEN
     printf("%s:%d, assertion %s: %s\n", filename, line, expression, message ? message : "failed" );
 #else
     REPORT_FATAL_ERROR("%s:%d, assertion %s: %s\n", filename, line, expression, message ? message : "failed" );
 #endif
+
     if( ErrorExtraCall )
         (*ErrorExtraCall)();
+    fflush(stdout); fflush(stderr);
 #if HARNESS_TERMINATE_ON_ASSERT
     TerminateProcess(GetCurrentProcess(), 1);
 #elif HARNESS_EXIT_ON_ASSERT
     exit(1);
+#elif HARNESS_CONTINUE_ON_ASSERT
+    // continue testing
+#elif _MSC_VER && _DEBUG
+    // aligned with tbb_assert_impl.h behavior
+    if(1 == _CrtDbgReport(_CRT_ASSERT, filename, line, NULL, "%s\r\n%s", expression, message?message:""))
+        _CrtDbgBreak();
 #else
     abort();
 #endif /* HARNESS_EXIT_ON_ASSERT */
@@ -138,10 +212,31 @@ void ReportError( const char* filename, int line, const char* expression, const
 void ReportWarning( const char* filename, int line, const char* expression, const char * message ) {
     REPORT("Warning: %s:%d, assertion %s: %s\n", filename, line, expression, message ? message : "failed" );
 }
-#else
-#define ASSERT(p,msg) ((void)0)
-#define ASSERT_WARNING(p,msg) ((void)0)
-#endif /* HARNESS_NO_ASSERT */
+
+#else /* !HARNESS_NO_ASSERT */
+
+#define ASSERT(p,msg) (Harness::suppress_unused_warning(p), (void)0)
+#define ASSERT_WARNING(p,msg) (Harness::suppress_unused_warning(p), (void)0)
+
+#endif /* !HARNESS_NO_ASSERT */
+
+//TODO: unify with utility::internal::array_length from examples common utilities
+template<typename T, size_t N>
+inline size_t array_length(const T(&)[N])
+{
+   return N;
+}
+
+//TODO: remove this #if __TBB_INITIALIZER_LISTS_PRESENT
+//it looks like all other compilers except gcc issue warnings/errors then they see
+//declaration of zero sized array
+#if __TBB_INITIALIZER_LISTS_PRESENT
+template<typename T>
+inline size_t array_length(const T[0])
+{
+   return 0;
+}
+#endif //__TBB_INITIALIZER_LISTS_PRESENT
 
 #if !HARNESS_NO_PARSE_COMMAND_LINE
 
@@ -214,16 +309,8 @@ static void ParseCommandLine( int argc, char* argv[] ) {
 }
 #endif /* HARNESS_NO_PARSE_COMMAND_LINE */
 
-#if HARNESS_USE_PROXY
-    #define TBB_PREVIEW_RUNTIME_LOADER 1
-    #include "tbb/runtime_loader.h"
-    static char const * _path[] = { ".", NULL };
-    static tbb::runtime_loader _runtime_loader( _path );
-#endif // HARNESS_USE_PROXY
-
 #if !HARNESS_CUSTOM_MAIN
 
-
 #if __TBB_MPI_INTEROP
 #undef SEEK_SET
 #undef SEEK_CUR
@@ -235,13 +322,13 @@ HARNESS_EXPORT
 #if HARNESS_NO_PARSE_COMMAND_LINE
 int main() {
 #if __TBB_MPI_INTEROP
-    MPI_Init(NULL,NULL); 
+    MPI_Init(NULL,NULL);
 #endif
 #else
 int main(int argc, char* argv[]) {
     ParseCommandLine( argc, argv );
 #if __TBB_MPI_INTEROP
-    MPI_Init(&argc,&argv); 
+    MPI_Init(&argc,&argv);
 #endif
 #endif
 #if __TBB_MPI_INTEROP
@@ -250,8 +337,8 @@ int main(int argc, char* argv[]) {
     // Master process receives this info and print it in verbose mode
     int rank, size, myrank;
     MPI_Status status;
-    MPI_Comm_size(MPI_COMM_WORLD,&size); 
-    MPI_Comm_rank(MPI_COMM_WORLD,&myrank); 
+    MPI_Comm_size(MPI_COMM_WORLD,&size);
+    MPI_Comm_rank(MPI_COMM_WORLD,&myrank);
     if (myrank == 0) {
 #if !HARNESS_NO_PARSE_COMMAND_LINE
         REMARK("Hello mpi world. I am %d of %d\n", myrank, size);
@@ -268,7 +355,20 @@ int main(int argc, char* argv[]) {
         MPI_Send (&size, 1, MPI_INT, 0, 1, MPI_COMM_WORLD);
     }
 #endif
+#if __TBB_MIC_OFFLOAD
+    int res = Harness::Unknown;
+    #pragma offload target(mic:-1) out(res)
+    {
+    #if __MIC__
+        res = TestMain ();
+    #else
+        ASSERT( 0, "Host execution in offload mode!" );
+        exit(1);
+    #endif
+    }
+#else
     int res = TestMain ();
+#endif
     ASSERT( res==Harness::Done || res==Harness::Skipped, "Wrong return code by TestMain");
 #if __TBB_MPI_INTEROP
     if (myrank == 0) {
@@ -281,7 +381,6 @@ int main(int argc, char* argv[]) {
     return 0;
 }
 
-
 #endif /* !HARNESS_CUSTOM_MAIN */
 
 //! Base class for prohibiting compiler-generated operator=
@@ -316,7 +415,13 @@ public:
     void start() {
 #if _WIN32||_WIN64
         unsigned thread_id;
+#if __TBB_WIN8UI_SUPPORT
+        std::thread* thread_tmp=new std::thread(thread_function, this);
+        thread_handle = thread_tmp->native_handle();
+        thread_id = 0;
+#else
         thread_handle = (HANDLE)_beginthreadex( NULL, 0, thread_function, this, 0, &thread_id );
+#endif
         ASSERT( thread_handle!=0, "NativeParallelFor: _beginthreadex failed" );
 #else
 #if __ICC==1100
@@ -327,8 +432,9 @@ public:
         // launched by make, the default stack size is set to the hard limit, and
         // calls to pthread_create fail with out-of-memory error.
         // Therefore we set the stack size explicitly (as for TBB worker threads).
-        const size_t MByte = 1<<20;
-#if __i386__||__i386
+// TODO: make a single definition of MByte used by all tests.
+        const size_t MByte = 1024*1024;
+#if __i386__||__i386||__arm__
         const size_t stack_size = 1*MByte;
 #elif __x86_64__
         const size_t stack_size = 2*MByte;
@@ -352,7 +458,7 @@ public:
     //! Wait for task to finish
     void wait_to_finish() {
 #if _WIN32||_WIN64
-        DWORD status = WaitForSingleObject( thread_handle, INFINITE );
+        DWORD status = WaitForSingleObjectEx( thread_handle, INFINITE, FALSE );
         ASSERT( status!=WAIT_FAILED, "WaitForSingleObject failed" );
         CloseHandle( thread_handle );
 #else
@@ -504,7 +610,15 @@ public:
 #endif /* !HARNESS_NO_ASSERT */
 
 #if _WIN32 || _WIN64
-    void Sleep ( int ms ) { ::Sleep(ms); }
+    void Sleep ( int ms ) {
+#if !__TBB_WIN8UI_SUPPORT
+        ::Sleep(ms);
+#else
+         std::chrono::milliseconds sleep_time( ms );
+         std::this_thread::sleep_for( sleep_time );
+#endif
+
+    }
 
     typedef DWORD tid_t;
     tid_t CurrentTid () { return GetCurrentThreadId(); }
