@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -26,27 +26,45 @@
     the GNU General Public License.
 */
 
-const int MByte = 1048576; //1MB
+const unsigned MByte = 1024*1024;
 bool __tbb_test_errno = false;
 
-/* _WIN32_WINNT should be defined at the very beginning, 
+#include "tbb/tbb_config.h"
+
+#if __TBB_WIN8UI_SUPPORT	
+// testing allocator itself not iterfaces
+// so we can use desktop functions
+#define _CRT_USE_WINAPI_FAMILY_DESKTOP_APP 1
+#define HARNESS_NO_PARSE_COMMAND_LINE 1
+#include "harness.h"
+// FIXME: fix the test to support New Windows *8 Store Apps mode.
+int TestMain() {
+    return Harness::Skipped;
+}
+#else /* __TBB_WIN8UI_SUPPORT	 */
+
+/* _WIN32_WINNT should be defined at the very beginning,
    because other headers might include <windows.h>
 */
 
 #if _WIN32 || _WIN64
 #undef _WIN32_WINNT
-#define _WIN32_WINNT 0x0500
-#include <windows.h>
+#define _WIN32_WINNT 0x0501
+#include "tbb/machine/windows_api.h"
 #include <stdio.h>
 #include "harness_report.h"
 
-void limitMem( int limit )
+#if _MSC_VER && defined(_MT) && defined(_DLL)
+    #pragma comment(lib, "version.lib")  // to use GetFileVersionInfo*
+#endif
+
+void limitMem( size_t limit )
 {
     static HANDLE hJob = NULL;
     JOBOBJECT_EXTENDED_LIMIT_INFORMATION jobInfo;
 
     jobInfo.BasicLimitInformation.LimitFlags = JOB_OBJECT_LIMIT_PROCESS_MEMORY;
-    jobInfo.ProcessMemoryLimit = limit? limit*MByte : 2*1024LL*MByte;
+    jobInfo.ProcessMemoryLimit = limit? limit*MByte : 2*MByte*1024;
     if (NULL == hJob) {
         if (NULL == (hJob = CreateJobObject(NULL, NULL))) {
             REPORT("Can't assign create job object: %ld\n", GetLastError());
@@ -57,7 +75,7 @@ void limitMem( int limit )
             exit(1);
         }
     }
-    if (0 == SetInformationJobObject(hJob, JobObjectExtendedLimitInformation, 
+    if (0 == SetInformationJobObject(hJob, JobObjectExtendedLimitInformation,
                                      &jobInfo, sizeof(jobInfo))) {
         REPORT("Can't set limits: %ld\n", GetLastError());
         exit(1);
@@ -72,7 +90,7 @@ void limitMem( int limit )
 #include <sys/types.h>  // uint64_t on FreeBSD, needed for rlim_t
 #include "harness_report.h"
 
-void limitMem( int limit )
+void limitMem( size_t limit )
 {
     rlimit rlim;
     rlim.rlim_cur = limit? limit*MByte : (rlim_t)RLIM_INFINITY;
@@ -83,7 +101,7 @@ void limitMem( int limit )
         exit(1);
     }
 }
-#endif 
+#endif
 
 #define ASSERT_ERRNO(cond, msg)  ASSERT( !__tbb_test_errno || (cond), msg )
 #define CHECK_ERRNO(cond) (__tbb_test_errno && (cond))
@@ -92,16 +110,21 @@ void limitMem( int limit )
 #include <errno.h>
 #define __TBB_NO_IMPLICIT_LINKAGE 1
 #include "tbb/scalable_allocator.h"
-#include "tbb/tbb_machine.h"
 
 #define HARNESS_CUSTOM_MAIN 1
 #include "harness.h"
 #include "harness_barrier.h"
+#include "harness_tbb_independence.h"
 #if __linux__
 #include <stdint.h> // uintptr_t
 #endif
 #if _WIN32 || _WIN64
 #include <malloc.h> // _aligned_(malloc|free|realloc)
+#if __MINGW64__
+// Workaround a bug in MinGW64 headers with _aligned_(malloc|free) not declared by default
+extern "C" void __cdecl _aligned_free(void *);
+extern "C" void *__cdecl _aligned_malloc(size_t,size_t);
+#endif
 #endif
 
 #if !TBB_USE_EXCEPTIONS && _MSC_VER
@@ -158,42 +181,20 @@ TestAlignedRealloc* Raligned_realloc;
 bool error_occurred = false;
 
 #if __APPLE__
-// Tests that use the variable are skipped on Mac OS* X
+// Tests that use the variable are skipped on OS X*
 #else
 static bool perProcessLimits = true;
 #endif
 
 const size_t POWERS_OF_2 = 20;
 
-#if __linux__  && __ia64__
-/* Can't use Intel compiler intrinsic due to internal error reported by
-   10.1 compiler */
-pthread_mutex_t counter_mutex = PTHREAD_MUTEX_INITIALIZER;
-
-int32_t __TBB_machine_fetchadd4__TBB_full_fence (volatile void *ptr, int32_t value)
-{
-    pthread_mutex_lock(&counter_mutex);
-    int32_t result = *(int32_t*)ptr;
-    *(int32_t*)ptr = result + value;
-    pthread_mutex_unlock(&counter_mutex);
-    return result;
-}
-
-void __TBB_machine_pause(int32_t /*delay*/) {}
-
-#elif (_WIN32||_WIN64) && defined(_M_AMD64)
-
-void __TBB_machine_pause(__int32 /*delay*/ ) {}
-
-#endif
-
 struct MemStruct
 {
     void* Pointer;
     UINT Size;
 
     MemStruct() : Pointer(NULL), Size(0) {}
-    MemStruct(void* Pointer, UINT Size) : Pointer(Pointer), Size(Size) {}
+    MemStruct(void* ptr, UINT sz) : Pointer(ptr), Size(sz) {}
 };
 
 class CMemTest: NoAssign
@@ -204,8 +205,8 @@ class CMemTest: NoAssign
     static bool firstTime;
 
 public:
-    CMemTest(Harness::SpinBarrier *limitBarrier, bool isVerbose=false) :
-        CountErrors(0), limitBarrier(limitBarrier)
+    CMemTest(Harness::SpinBarrier *barrier, bool isVerbose=false) :
+        CountErrors(0), limitBarrier(barrier)
         {
             srand((UINT)time(NULL));
             FullLog=isVerbose;
@@ -216,7 +217,7 @@ public:
     void UniquePointer(); // unique pointer - check with padding
     void AddrArifm(); // unique pointer - check with pointer arithmetic
     bool ShouldReportError();
-    void Free_NULL(); // 
+    void Free_NULL(); //
     void Zerofilling(); // check if arrays are zero-filled
     void TestAlignedParameters();
     void RunAllTests(int total_threads);
@@ -224,9 +225,9 @@ public:
 };
 
 class Limit {
-    int limit;
+    size_t limit;
 public:
-    Limit(int limit) : limit(limit) {}
+    Limit(size_t a_limit) : limit(a_limit) {}
     void operator() () const {
         limitMem(limit);
     }
@@ -241,7 +242,7 @@ struct RoundRobin: NoAssign {
 
     RoundRobin( long p, Harness::SpinBarrier *limitBarrier, bool verbose ) :
         number_of_threads(p), test(limitBarrier, verbose) {}
-    void operator()( int /*id*/ ) const 
+    void operator()( int /*id*/ ) const
         {
             test.RunAllTests(number_of_threads);
         }
@@ -260,12 +261,13 @@ static void setSystemAllocs()
     Raligned_realloc=_aligned_realloc;
     Taligned_free=_aligned_free;
     Rposix_memalign=0;
-#elif  __APPLE__ || __sun //  Max OS X and Solaris don't have posix_memalign
+#elif  __APPLE__ || __sun || __ANDROID__ 
+// OS X*, Solaris, and Android don't have posix_memalign
     Raligned_malloc=0;
     Raligned_realloc=0;
     Taligned_free=0;
     Rposix_memalign=0;
-#else 
+#else
     Raligned_malloc=0;
     Raligned_realloc=0;
     Taligned_free=0;
@@ -282,7 +284,7 @@ void ReallocParam()
 
     bufs[0] = Trealloc(NULL, 30*MByte);
     ASSERT(bufs[0], "Can't get memory to start the test.");
-  
+
     for (i=1; i<ITERS; i++)
     {
         bufs[i] = Trealloc(NULL, 30*MByte);
@@ -290,14 +292,14 @@ void ReallocParam()
             break;
     }
     ASSERT(i<ITERS, "Limits should be decreased for the test to work.");
-  
+
     Trealloc(bufs[0], 0);
-    /* There is a race for the free space between different threads at 
+    /* There is a race for the free space between different threads at
        this point. So, have to run the test sequentially.
     */
     bufs[0] = Trealloc(NULL, 30*MByte);
     ASSERT(bufs[0], NULL);
-  
+
     for (int j=0; j<i; j++)
         Trealloc(bufs[j], 0);
 }
@@ -328,29 +330,28 @@ int main(int argc, char* argv[]) {
 
     ParseCommandLine( argC, argV );
 #if __linux__
-    /* According to man pthreads 
+    /* According to man pthreads
        "NPTL threads do not share resource limits (fixed in kernel 2.6.10)".
        Use per-threads limits for affected systems.
      */
     if ( LinuxKernelVersion() < 2*1000000 + 6*1000 + 10)
         perProcessLimits = false;
-#endif    
+#endif
     //-------------------------------------
 #if __APPLE__
-    /* Skip due to lack of memory limit enforcing under Mac OS X. */
+    /* Skip due to lack of memory limit enforcing under OS X*. */
 #else
     limitMem(200);
     ReallocParam();
     limitMem(0);
 #endif
-    
-//for linux and dynamic runtime errno is used to check allocator fuctions
+
+//for linux and dynamic runtime errno is used to check allocator functions
 //check if library compiled with /MD(d) and we can use errno
-#if _MSC_VER 
+#if _MSC_VER
 #if defined(_MT) && defined(_DLL) //check errno if test itself compiled with /MD(d) only
-    #pragma comment(lib, "version.lib")
     char*  version_info_block = NULL;
-    int version_info_block_size; 
+    int version_info_block_size;
     LPVOID comments_block = NULL;
     UINT comments_block_size;
 #ifdef _DEBUG
@@ -359,7 +360,7 @@ int main(int argc, char* argv[]) {
 #define __TBBMALLOCDLL "tbbmalloc.dll"
 #endif //_DEBUG
     version_info_block_size = GetFileVersionInfoSize( __TBBMALLOCDLL, (LPDWORD)&version_info_block_size );
-    if( version_info_block_size 
+    if( version_info_block_size
         && ((version_info_block = (char*)malloc(version_info_block_size)) != NULL)
         && GetFileVersionInfo(  __TBBMALLOCDLL, NULL, version_info_block_size, version_info_block )
         && VerQueryValue( version_info_block, "\\StringFileInfo\\000004b0\\Comments", &comments_block, &comments_block_size )
@@ -379,11 +380,32 @@ int main(int argc, char* argv[]) {
         NativeParallelFor( p, RoundRobin(p, barrier, Verbose) );
         delete barrier;
     }
-    if( !error_occurred ) 
+    if( !error_occurred )
         REPORT("done\n");
     return 0;
 }
 
+// if non-zero byte found, returns bad value address plus 1
+size_t NonZero(void *ptr, size_t size)
+{
+    size_t words = size / sizeof(intptr_t);
+    size_t tailSz = size % sizeof(intptr_t);
+    intptr_t *buf =(intptr_t*)ptr;
+    char *bufTail =(char*)(buf+words);
+
+    for (size_t i=0; i<words; i++)
+        if (buf[i]) {
+            for (unsigned b=0; b<sizeof(intptr_t); b++)
+                if (((char*)(buf+i))[b])
+                    return sizeof(intptr_t)*i + b + 1;
+        }
+    for (size_t i=0; i<tailSz; i++)
+        if (bufTail[i]) {
+            return words*sizeof(intptr_t)+i+1;
+        }
+    return 0;
+}
+
 struct TestStruct
 {
     DWORD field1:2;
@@ -391,22 +413,8 @@ struct TestStruct
     double field3;
     UCHAR field4[100];
     TestStruct* field5;
-//  std::string field6;
     std::vector<int> field7;
     double field8;
-    bool IsZero() {
-        int wordSz = sizeof(TestStruct) / sizeof(intptr_t);
-        int tailSz = sizeof(TestStruct) % sizeof(intptr_t);
-
-        intptr_t *buf =(intptr_t*)this;
-        char *bufTail =(char*) (buf+wordSz);
-
-        for (int i=0; i<wordSz; i++)
-            if (buf[i]) return false;
-        for (int i=0; i<tailSz; i++)
-            if (bufTail[i]) return false;
-        return true;
-    }
 };
 
 int Tposix_memalign(void **memptr, size_t alignment, size_t size)
@@ -580,7 +588,7 @@ void CMemTest::Zerofilling()
             continue;
         for (size_t j=0; j<CountElement; j++)
         {
-            if (!(TSMas+j)->IsZero())
+            if (NonZero(TSMas+j, sizeof(TestStruct)))
             {
                 CountErrors++;
                 if (ShouldReportError()) REPORT("detect nonzero element at TestStruct\n");
@@ -594,10 +602,31 @@ void CMemTest::Zerofilling()
 }
 
 #if !__APPLE__
+
+void myMemset(void *ptr, int c, size_t n)
+{
+#if  __linux__ &&  __i386__
+// memset in Fedora 13 not always correctly sets memory to required values.
+    char *p = (char*)ptr;
+    for (size_t i=0; i<n; i++)
+        p[i] = c;
+#else
+    memset(ptr, c, n);
+#endif
+}
+
+// This test requires more than TOTAL_MB_ALLOC MB of RAM.
+#if __ANDROID__
+// Android requires lower limit due to lack of virtual memory.
+#define TOTAL_MB_ALLOC	200
+#else
+#define TOTAL_MB_ALLOC  800
+#endif
 void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
 {
+    const int MB_PER_THREAD = TOTAL_MB_ALLOC / total_threads;
     // find size to guarantee getting NULL for 1024 B allocations
-    const int MAXNUM_1024 = (200+50)*1024;
+    const int MAXNUM_1024 = (MB_PER_THREAD + (MB_PER_THREAD>>2)) * 1024;
 
     std::vector<MemStruct> PointerList;
     void *tmp;
@@ -605,27 +634,28 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
     int CountNULL, num_1024;
     if (FullLog) REPORT("\nNULL return & check errno:\n");
     UINT Size;
-    Limit limit_200M(200*total_threads), no_limit(0);
+    Limit limit_total(TOTAL_MB_ALLOC), no_limit(0);
     void **buf_1024 = (void**)Tmalloc(MAXNUM_1024*sizeof(void*));
 
     ASSERT(buf_1024, NULL);
-    /* We must have space for pointers when memory limit is hit. 
-       Reserve enough for the worst case. 
+    /* We must have space for pointers when memory limit is hit.
+       Reserve enough for the worst case, taking into account race for
+       limited space between threads.
     */
-    PointerList.reserve(200*MByte/MinSize);
+    PointerList.reserve(TOTAL_MB_ALLOC*MByte/MinSize);
 
-    /* There is a bug in the specific verion of GLIBC (2.5-12) shipped 
-       with RHEL5 that leads to erroneous working of the test 
+    /* There is a bug in the specific version of GLIBC (2.5-12) shipped
+       with RHEL5 that leads to erroneous working of the test
        on Intel64 and IPF systems when setrlimit-related part is enabled.
        Switching to GLIBC 2.5-18 from RHEL5.1 resolved the issue.
      */
     if (perProcessLimits)
-        limitBarrier->wait(limit_200M);
+        limitBarrier->wait(limit_total);
     else
-        limitMem(200);
+        limitMem(MB_PER_THREAD);
 
-    /* regression test against the bug in allocator when it dereference NULL 
-       while lack of memory 
+    /* regression test against the bug in allocator when it dereference NULL
+       while lack of memory
     */
     for (num_1024=0; num_1024<MAXNUM_1024; num_1024++) {
         buf_1024[num_1024] = Tcalloc(1024, 1);
@@ -643,7 +673,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
         tmp=Tmalloc(Size);
         if (tmp != NULL)
         {
-            memset(tmp, 0, Size);
+            myMemset(tmp, 0, Size);
             PointerList.push_back(MemStruct(tmp, Size));
         }
     } while(tmp != NULL);
@@ -680,7 +710,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
                     CountErrors++;
                     if (ShouldReportError()) REPORT("error: errno changed to %d though valid pointer was returned\n", errno);
                 }
-                memset(tmp, 0, Size);
+                myMemset(tmp, 0, Size);
                 PointerList.push_back(MemStruct(tmp, Size));
             }
         }
@@ -698,7 +728,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
         {
             Size=rand()%(MaxSize-MinSize)+MinSize;
             errno = ENOMEM+j+1;
-            tmp=Tcalloc(COUNT_ELEM_CALLOC,Size);  
+            tmp=Tcalloc(COUNT_ELEM_CALLOC,Size);
             if (tmp == NULL)
             {
                 CountNULL++;
@@ -718,7 +748,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
                 if ( CHECK_ERRNO(errno != ENOMEM+j+1) && !known_issue ) {
                     CountErrors++;
                     if (ShouldReportError()) REPORT("error: errno changed to %d though valid pointer was returned\n", errno);
-                }      
+                }
                 PointerList.push_back(MemStruct(tmp, Size));
             }
         }
@@ -744,7 +774,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
                     if (errno != 0 && !known_issue) {
                         CountErrors++;
                         if (ShouldReportError()) REPORT("valid pointer returned, error: errno not kept\n");
-                    }      
+                    }
                     PointerList[i].Size *= 2;
                 }
                 else if (tmp != PointerList[i].Pointer && tmp != NULL) // another place
@@ -758,7 +788,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
                         if (ShouldReportError()) REPORT("valid pointer returned, error: errno not kept\n");
                     }
                     // newly allocated area have to be zeroed
-                    memset((char*)tmp + PointerList[i].Size, 0, PointerList[i].Size);
+                    myMemset((char*)tmp + PointerList[i].Size, 0, PointerList[i].Size);
                     PointerList[i].Pointer = tmp;
                     PointerList[i].Size *= 2;
                 }
@@ -771,13 +801,10 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
                         if (ShouldReportError()) REPORT("NULL returned, error: errno(%d) != ENOMEM\n", errno);
                     }
                     // check data integrity
-                    BYTE *zer=(BYTE*)PointerList[i].Pointer;
-                    for (UINT k=0; k<PointerList[i].Size; k++)
-                        if (zer[k] != 0)
-                        {
-                            CountErrors++;
-                            if (ShouldReportError()) REPORT("NULL returned, error: data changed\n");
-                        }
+                    if (NonZero(PointerList[i].Pointer, PointerList[i].Size)) {
+                        CountErrors++;
+                        if (ShouldReportError()) REPORT("NULL returned, error: data changed\n");
+                    }
                 }
             }
     if (FullLog) REPORT("realloc end\n");
@@ -812,21 +839,17 @@ void CMemTest::UniquePointer()
         MasPointer[i]=(int*)Tmalloc(MasCountElem[i]*sizeof(int));
         if (NULL == MasPointer[i])
             MasCountElem[i]=0;
-        for (UINT j=0; j<MasCountElem[i]; j++)
-            *(MasPointer[i]+j)=0;
+        memset(MasPointer[i], 0, sizeof(int)*MasCountElem[i]);
     }
     if (FullLog) REPORT("malloc....");
     for (UINT i=0; i<COUNT_ELEM-1; i++)
     {
-        for (UINT j=0; j<MasCountElem[i]; j++)
-        {
-            if (*(*(MasPointer+i)+j)!=0)
-            {
-                CountErrors++;
-                if (ShouldReportError()) REPORT("error, detect 1 with 0x%p\n",(*(MasPointer+i)+j));
-            }
-            *(*(MasPointer+i)+j)+=1;
+        if (size_t badOff = NonZero(MasPointer[i], sizeof(int)*MasCountElem[i])) {
+            CountErrors++;
+            if (ShouldReportError())
+                REPORT("error, detect non-zero at %p\n", (char*)MasPointer[i]+badOff-1);
         }
+        memset(MasPointer[i], 1, sizeof(int)*MasCountElem[i]);
     }
     if (CountErrors) REPORT("%s\n",strError);
     else if (FullLog) REPORT("%s\n",strOk);
@@ -845,15 +868,12 @@ void CMemTest::UniquePointer()
     if (FullLog) REPORT("calloc....");
     for (int i=0; i<COUNT_ELEM-1; i++)
     {
-        for (UINT j=0; j<*(MasCountElem+i); j++)
-        {
-            if (*(*(MasPointer+i)+j)!=0)
-            {
-                CountErrors++;
-                if (ShouldReportError()) REPORT("error, detect 1 with 0x%p\n",(*(MasPointer+i)+j));
-            }
-            *(*(MasPointer+i)+j)+=1;
+        if (size_t badOff = NonZero(MasPointer[i], sizeof(int)*MasCountElem[i])) {
+            CountErrors++;
+            if (ShouldReportError())
+                REPORT("error, detect non-zero at %p\n", (char*)MasPointer[i]+badOff-1);
         }
+        memset(MasPointer[i], 1, sizeof(int)*MasCountElem[i]);
     }
     if (CountErrors) REPORT("%s\n",strError);
     else if (FullLog) REPORT("%s\n",strOk);
@@ -868,20 +888,14 @@ void CMemTest::UniquePointer()
             (int*)Trealloc(*(MasPointer+i),MasCountElem[i]*sizeof(int));
         if (NULL == MasPointer[i])
             MasCountElem[i]=0;
-        for (UINT j=0; j<MasCountElem[i]; j++)
-            *(*(MasPointer+i)+j)=0;
+        memset(MasPointer[i], 0, sizeof(int)*MasCountElem[i]);
     }
     if (FullLog) REPORT("realloc....");
     for (int i=0; i<COUNT_ELEM-1; i++)
     {
-        for (UINT j=0; j<*(MasCountElem+i); j++)
-        {
-            if (*(*(MasPointer+i)+j)!=0)
-            {
-                CountErrors++;
-            }
-            *(*(MasPointer+i)+j)+=1;
-        }
+        if (NonZero(MasPointer[i], sizeof(int)*MasCountElem[i]))
+            CountErrors++;
+        memset(MasPointer[i], 1, sizeof(int)*MasCountElem[i]);
     }
     if (CountErrors) REPORT("%s\n",strError);
     else if (FullLog) REPORT("%s\n",strOk);
@@ -935,13 +949,13 @@ void CMemTest::TestAlignedParameters()
                 ret = Tposix_memalign(NULL, bad_align, 100);
                 ASSERT(EINVAL==ret, NULL);
             }
-    
+
         memptr = &ret;
         ret = Tposix_memalign(&memptr, 5*sizeof(void*), 100);
         ASSERT(memptr == &ret,
-               "memptr should not be changed after unsuccesful call");
+               "memptr should not be changed after unsuccessful call");
         ASSERT(EINVAL==ret, NULL);
-    
+
         // alignment is power of 2, but not a multiple of sizeof(void *),
         // we expect that sizeof(void*) > 2
         ret = Tposix_memalign(NULL, 2, 100);
@@ -955,7 +969,7 @@ void CMemTest::TestAlignedParameters()
                 ASSERT(NULL==memptr, NULL);
                 ASSERT_ERRNO(EINVAL==errno, NULL);
             }
-    
+
         // size is zero
         memptr = Taligned_malloc(0, 16);
         ASSERT(NULL==memptr, "size is zero, so must return NULL");
@@ -965,7 +979,7 @@ void CMemTest::TestAlignedParameters()
         // NULL pointer is OK to free
         errno = 0;
         Taligned_free(NULL);
-        /* As there is no return value for free, strictly speaking we can't 
+        /* As there is no return value for free, strictly speaking we can't
            check errno here. But checked implementations obey the assertion.
         */
         ASSERT_ERRNO(0==errno, NULL);
@@ -1003,13 +1017,16 @@ void CMemTest::RunAllTests(int total_threads)
         InvariantDataRealloc(/*aligned=*/true);
     TestAlignedParameters();
 #if __APPLE__
-    REPORT("Known issue: some tests are skipped on Mac OS* X\n");
+    REPORT("Known issue: some tests are skipped on OS X*\n");
 #else
     UniquePointer();
     AddrArifm();
-#if !__TBB_LRB_NATIVE
+#if !__TBB_MIC_NATIVE
     NULLReturn(1*MByte,100*MByte,total_threads);
 #endif
 #endif
-    if (FullLog) REPORT("All tests ended\nclearing memory...");
+    if (FullLog) REPORT("Tests for %d threads ended\n", total_threads);
 }
+
+#endif /* __TBB_WIN8UI_SUPPORT	 */
+
