@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -26,6 +26,14 @@
     the GNU General Public License.
 */
 
+/* to prevent loading dynamic TBBmalloc at startup, that is not needed
+   for the whitebox test */
+#define __TBB_SOURCE_DIRECTLY_INCLUDED 1
+
+// According to C99 standard INTPTR_MIN defined for C++
+// iff __STDC_LIMIT_MACROS pre-defined
+#define __STDC_LIMIT_MACROS 1
+
 #include "harness.h"
 #include "harness_barrier.h"
 
@@ -34,6 +42,10 @@
 #undef DO_ITT_NOTIFY
 #endif
 
+#define __TBB_MALLOC_WHITEBOX_TEST 1 // to get access to LOC internals
+// help trigger rare race condition
+#define WhiteboxTestingYield() (__TBB_Yield(), __TBB_Yield(), __TBB_Yield(), __TBB_Yield())
+
 #define protected public
 #define private public
 #include "../tbbmalloc/frontend.cpp"
@@ -53,8 +65,8 @@ class AllocInfo {
     int size;
 public:
     AllocInfo() : p(NULL), val(0), size(0) {}
-    explicit AllocInfo(int size) : p((int*)scalable_malloc(size*sizeof(int))),
-                                   val(rand()), size(size) {
+    explicit AllocInfo(int sz) : p((int*)scalable_malloc(sz*sizeof(int))),
+                                   val(rand()), size(sz) {
         ASSERT(p, NULL);
         for (int k=0; k<size; k++)
             p[k] = val;
@@ -68,12 +80,18 @@ public:
     }
 };
 
-class TestLargeObjCache: NoAssign {
+class SimpleBarrier: NoAssign {
+protected:
     static Harness::SpinBarrier barrier;
 public:
-    static int largeMemSizes[LARGE_MEM_SIZES_NUM];
-
     static void initBarrier(unsigned thrds) { barrier.initialize(thrds); }
+};
+
+Harness::SpinBarrier SimpleBarrier::barrier;
+
+class TestLargeObjCache: public SimpleBarrier {
+public:
+    static int largeMemSizes[LARGE_MEM_SIZES_NUM];
 
     TestLargeObjCache( ) {}
 
@@ -83,7 +101,7 @@ public:
         // push to maximal cache limit
         for (int i=0; i<2; i++) {
             const int sizes[] = { MByte/sizeof(int),
-                                  (MByte-2*largeBlockCacheStep)/sizeof(int) };
+                                  (MByte-2*LargeObjectCache::largeBlockCacheStep)/sizeof(int) };
             for (int q=0; q<2; q++) {
                 size_t curr = 0;
                 for (int j=0; j<LARGE_MEM_SIZES_NUM; j++, curr++)
@@ -117,13 +135,23 @@ public:
     }
 };
 
-Harness::SpinBarrier TestLargeObjCache::barrier;
 int TestLargeObjCache::largeMemSizes[LARGE_MEM_SIZES_NUM];
 
+void TestLargeObjectCache()
+{
+    for (int i=0; i<LARGE_MEM_SIZES_NUM; i++)
+        TestLargeObjCache::largeMemSizes[i] = 
+            (int)(minLargeObjectSize + 2*minLargeObjectSize*(1.*rand()/RAND_MAX));
+
+    for( int p=MaxThread; p>=MinThread; --p ) {
+        TestLargeObjCache::initBarrier( p );
+        NativeParallelFor( p, TestLargeObjCache() );
+    }
+}
+
 #if MALLOC_CHECK_RECURSION
 
-class TestStartupAlloc: NoAssign {
-    static Harness::SpinBarrier init_barrier;
+class TestStartupAlloc: public SimpleBarrier {
     struct TestBlock {
         void *ptr;
         size_t sz;
@@ -131,11 +159,10 @@ class TestStartupAlloc: NoAssign {
     static const int ITERS = 100;
 public:
     TestStartupAlloc() {}
-    static void initBarrier(unsigned thrds) { init_barrier.initialize(thrds); }
     void operator()(int) const {
         TestBlock blocks1[ITERS], blocks2[ITERS];
 
-        init_barrier.wait();
+        barrier.wait();
 
         for (int i=0; i<ITERS; i++) {
             blocks1[i].sz = rand() % minLargeObjectSize;
@@ -153,20 +180,18 @@ public:
 
             for (size_t j=0; j<blocks1[i].sz; j++)
                 ASSERT(*((char*)blocks1[i].ptr+j) == i, NULL);
-            Block *block = (Block *)alignDown(blocks1[i].ptr, blockSize);
+            Block *block = (Block *)alignDown(blocks1[i].ptr, slabSize);
             ((StartupBlock *)block)->free(blocks1[i].ptr);
         }
         for (int i=ITERS-1; i>=0; i--) {
             for (size_t j=0; j<blocks2[i].sz; j++)
                 ASSERT(*((char*)blocks2[i].ptr+j) == i, NULL);
-            Block *block = (Block *)alignDown(blocks2[i].ptr, blockSize);
+            Block *block = (Block *)alignDown(blocks2[i].ptr, slabSize);
             ((StartupBlock *)block)->free(blocks2[i].ptr);
         }
     }
 };
 
-Harness::SpinBarrier TestStartupAlloc::init_barrier;
-
 #endif /* MALLOC_CHECK_RECURSION */
 
 class BackRefWork: NoAssign {
@@ -206,9 +231,10 @@ public:
             scalable_free(objs[i]);
 
 #ifdef USE_WINTHREAD
-        // under Windows DllMain used to call mallocThreadShutdownNotification,
-        // as we don't use it have to call the callback manually
-        mallocThreadShutdownNotification(NULL);
+        // Under Windows DllMain is used for mallocThreadShutdownNotification
+        // calling. As DllMain is not used during whitebox testing,
+        // we have to call the callback manually.
+        __TBB_mallocThreadShutdownNotification();
 #endif
     }
 };
@@ -226,6 +252,53 @@ static void cleanObjectCache()
     defaultMemPool->extMemPool.hardCachesCleanup();
 }
 
+class TestInvalidBackrefs: public SimpleBarrier {
+#if __ANDROID__
+    // Android requires lower iters due to lack of virtual memory.
+    static const int BACKREF_GROWTH_ITERS = 50*1024;
+#else
+    static const int BACKREF_GROWTH_ITERS = 200*1024;
+#endif
+
+    static tbb::atomic<bool> backrefGrowthDone;
+    static void *ptrs[BACKREF_GROWTH_ITERS];
+public:
+    TestInvalidBackrefs() {}
+    void operator()(int id) const {
+
+        if (!id) {
+            backrefGrowthDone = false;
+            barrier.wait();
+
+            for (int i=0; i<BACKREF_GROWTH_ITERS; i++)
+                ptrs[i] = scalable_malloc(minLargeObjectSize);
+            backrefGrowthDone = true;
+            for (int i=0; i<BACKREF_GROWTH_ITERS; i++)
+                scalable_free(ptrs[i]);
+        } else {
+            void *p2 = scalable_malloc(minLargeObjectSize-1);
+            char *p1 = (char*)scalable_malloc(minLargeObjectSize-1);
+            LargeObjectHdr *hdr =
+                (LargeObjectHdr*)(p1+minLargeObjectSize-1 - sizeof(LargeObjectHdr));
+            hdr->backRefIdx.master = 7;
+            hdr->backRefIdx.largeObj = 1;
+            hdr->backRefIdx.offset = 2000;
+
+            barrier.wait();
+
+            while (!backrefGrowthDone) {
+                scalable_free(p2);
+                p2 = scalable_malloc(minLargeObjectSize-1);
+            }
+            scalable_free(p1);
+            scalable_free(p2);
+        }
+    }
+};
+
+tbb::atomic<bool> TestInvalidBackrefs::backrefGrowthDone;
+void *TestInvalidBackrefs::ptrs[BACKREF_GROWTH_ITERS];
+
 void TestBackRef() {
     size_t beforeNumBackRef, afterNumBackRef;
 
@@ -248,6 +321,14 @@ void TestBackRef() {
     NativeParallelFor( 1, FreeBlockPoolHit() );
     afterNumBackRef = allocatedBackRefCount();
     ASSERT(beforeNumBackRef==afterNumBackRef, "backreference leak detected");
+
+    // This is a regression test against race condition between backreference
+    // extension and checking invalid BackRefIdx.
+    // While detecting is object large or small, scalable_free 1st check for
+    // large objects, so there is a chance to prepend small object with
+    // seems valid BackRefIdx for large objects, and thus trigger the bug.
+    TestInvalidBackrefs::initBarrier(MaxThread);
+    NativeParallelFor( MaxThread, TestInvalidBackrefs() );
 }
 
 void *getMem(intptr_t /*pool_id*/, size_t &bytes)
@@ -297,26 +378,27 @@ int putMallocMem(intptr_t /*pool_id*/, void *ptr, size_t bytes)
 }
 
 void TestPools() {
-    rml::MemPoolPolicy pol;
+    rml::MemPoolPolicy pol(getMem, putMem);
     size_t beforeNumBackRef, afterNumBackRef;
 
-    pol.pAlloc = getMem;
-    pol.pFree = putMem;
-    pol.granularity = 8;
-    rml::MemoryPool *pool1 = pool_create(0, &pol);
-    rml::MemoryPool *pool2 = pool_create(0, &pol);
+    rml::MemoryPool *pool1;
+    rml::MemoryPool *pool2;
+    pool_create_v1(0, &pol, &pool1);
+    pool_create_v1(0, &pol, &pool2);
     pool_destroy(pool1);
     pool_destroy(pool2);
 
     cleanObjectCache();
     beforeNumBackRef = allocatedBackRefCount();
-    rml::MemoryPool *fixedPool = pool_create(0, &pol);
+    rml::MemoryPool *fixedPool;
 
+    pool_create_v1(0, &pol, &fixedPool);
     pol.pAlloc = getMallocMem;
     pol.pFree = putMallocMem;
     pol.granularity = 8;
-    rml::MemoryPool *mallocPool = pool_create(0, &pol);
+    rml::MemoryPool *mallocPool;
 
+    pool_create_v1(0, &pol, &mallocPool);
 /* check that large object cache (LOC) returns correct size for cached objects
    passBackendSz Byte objects are cached in LOC, but bypassed the backend, so
    memory requested directly from allocation callback.
@@ -326,7 +408,7 @@ void TestPools() {
    from LOC during LOC cleanup, and putMallocMem checks that returned size
    is correct.
 */
-    const size_t passBackendSz = Backend::maxBinedSize+1,
+    const size_t passBackendSz = Backend::maxBinned_HugePage+1,
         anotherLOCBinSz = minLargeObjectSize+1;
     for (int i=0; i<10; i++) { // run long enough to be cached
         void *p = pool_malloc(mallocPool, passBackendSz);
@@ -342,20 +424,25 @@ void TestPools() {
     }
 
     void *smallObj =  pool_malloc(fixedPool, 10);
+    ASSERT(smallObj, "Memory was not allocated");
     memset(smallObj, 1, 10);
     void *ptr = pool_malloc(fixedPool, 1024);
+    ASSERT(ptr, "Memory was not allocated");
     memset(ptr, 1, 1024);
     void *largeObj = pool_malloc(fixedPool, minLargeObjectSize);
+    ASSERT(largeObj, "Memory was not allocated");
     memset(largeObj, 1, minLargeObjectSize);
     ptr = pool_malloc(fixedPool, minLargeObjectSize);
+    ASSERT(ptr, "Memory was not allocated");
     memset(ptr, minLargeObjectSize, minLargeObjectSize);
     pool_malloc(fixedPool, 10*minLargeObjectSize); // no leak for unsuccesful allocations
     pool_free(fixedPool, smallObj);
     pool_free(fixedPool, largeObj);
 
     // provoke large object cache cleanup and hope no leaks occurs
-    for (size_t sz=minLargeObjectSize; sz<1*1024*1024; sz+=largeBlockCacheStep) {
+    for (size_t sz=minLargeObjectSize; sz<1*1024*1024; sz+=LargeObjectCache::largeBlockCacheStep) {
         ptr = pool_malloc(mallocPool, sz);
+        ASSERT(ptr, "Memory was not allocated");
         memset(ptr, sz, sz);
         pool_free(mallocPool, ptr);
     }
@@ -365,6 +452,57 @@ void TestPools() {
     cleanObjectCache();
     afterNumBackRef = allocatedBackRefCount();
     ASSERT(beforeNumBackRef==afterNumBackRef, "backreference leak detected");
+
+    {
+        // test usedSize/cachedSize and LOC bitmask correctness
+        void *p[5];
+        pool_create_v1(0, &pol, &mallocPool);
+        const LargeObjectCache *loc = &((rml::internal::MemoryPool*)mallocPool)->extMemPool.loc;
+        p[3] = pool_malloc(mallocPool, minLargeObjectSize+2*LargeObjectCache::largeBlockCacheStep);
+        for (int i=0; i<10; i++) {
+            p[0] = pool_malloc(mallocPool, minLargeObjectSize);
+            p[1] = pool_malloc(mallocPool, minLargeObjectSize+LargeObjectCache::largeBlockCacheStep);
+            pool_free(mallocPool, p[0]);
+            pool_free(mallocPool, p[1]);
+        }
+        ASSERT(loc->getUsedSize(), NULL);
+        pool_free(mallocPool, p[3]);
+        ASSERT(loc->getLOCSize() < 3*(minLargeObjectSize+LargeObjectCache::largeBlockCacheStep), NULL);
+        const size_t maxLocalLOCSize = LocalLOC<3,30>::getMaxSize();
+        ASSERT(loc->getUsedSize() <= maxLocalLOCSize, NULL);
+        for (int i=0; i<3; i++)
+            p[i] = pool_malloc(mallocPool, minLargeObjectSize+i*LargeObjectCache::largeBlockCacheStep);
+        size_t currUser = loc->getUsedSize();
+        ASSERT(!loc->getLOCSize() && currUser >= 3*(minLargeObjectSize+LargeObjectCache::largeBlockCacheStep), NULL);
+        p[4] = pool_malloc(mallocPool, minLargeObjectSize+3*LargeObjectCache::largeBlockCacheStep);
+        ASSERT(loc->getUsedSize() - currUser >= minLargeObjectSize+3*LargeObjectCache::largeBlockCacheStep, NULL);
+        pool_free(mallocPool, p[4]);
+        ASSERT(loc->getUsedSize() <= currUser+maxLocalLOCSize, NULL);
+        pool_reset(mallocPool);
+        ASSERT(!loc->getLOCSize() && !loc->getUsedSize(), NULL);
+        pool_destroy(mallocPool);
+    }
+    // To test LOC we need bigger lists than released by current LocalLOC
+    //   in production code. Create special LocalLOC.
+    {
+        LocalLOC<2, 20> lLOC;
+        pool_create_v1(0, &pol, &mallocPool);
+        rml::internal::ExtMemoryPool *mPool = &((rml::internal::MemoryPool*)mallocPool)->extMemPool;
+        const LargeObjectCache *loc = &((rml::internal::MemoryPool*)mallocPool)->extMemPool.loc;
+        for (int i=0; i<22; i++) {
+            void *o = pool_malloc(mallocPool, minLargeObjectSize+i*LargeObjectCache::largeBlockCacheStep);
+            bool ret = lLOC.put(((LargeObjectHdr*)o - 1)->memoryBlock, mPool);
+            ASSERT(ret, NULL);
+
+            o = pool_malloc(mallocPool, minLargeObjectSize+i*LargeObjectCache::largeBlockCacheStep);
+            ret = lLOC.put(((LargeObjectHdr*)o - 1)->memoryBlock, mPool);
+            ASSERT(ret, NULL);
+        }
+        lLOC.clean(mPool);
+        ASSERT(!loc->getUsedSize(), NULL);
+
+        pool_destroy(mallocPool);
+    }
 }
 
 void TestObjectRecognition() {
@@ -375,22 +513,24 @@ void TestObjectRecognition() {
     ASSERT(sizeof(BackRefIdx)==4, "Unexpected size of BackRefIdx");
     ASSERT(getObjectSize(falseObjectSize)!=falseObjectSize, "Error in test: bad choice for false object size");
 
-    void* mem = scalable_malloc(2*blockSize);
-    Block* falseBlock = (Block*)alignUp((uintptr_t)mem, blockSize);
+    void* mem = scalable_malloc(2*slabSize);
+    ASSERT(mem, "Memory was not allocated");
+    Block* falseBlock = (Block*)alignUp((uintptr_t)mem, slabSize);
     falseBlock->objectSize = falseObjectSize;
     char* falseSO = (char*)falseBlock + falseObjectSize*7;
-    ASSERT(alignDown(falseSO, blockSize)==(void*)falseBlock, "Error in test: false object offset is too big");
+    ASSERT(alignDown(falseSO, slabSize)==(void*)falseBlock, "Error in test: false object offset is too big");
 
-    void* bufferLOH = scalable_malloc(2*blockSize + headersSize);
+    void* bufferLOH = scalable_malloc(2*slabSize + headersSize);
+    ASSERT(bufferLOH, "Memory was not allocated");
     LargeObjectHdr* falseLO = 
-        (LargeObjectHdr*)alignUp((uintptr_t)bufferLOH + headersSize, blockSize);
+        (LargeObjectHdr*)alignUp((uintptr_t)bufferLOH + headersSize, slabSize);
     LargeObjectHdr* headerLO = (LargeObjectHdr*)falseLO-1;
     headerLO->memoryBlock = (LargeMemoryBlock*)bufferLOH;
-    headerLO->memoryBlock->unalignedSize = 2*blockSize + headersSize;
-    headerLO->memoryBlock->objectSize = blockSize + headersSize;
+    headerLO->memoryBlock->unalignedSize = 2*slabSize + headersSize;
+    headerLO->memoryBlock->objectSize = slabSize + headersSize;
     headerLO->backRefIdx = BackRefIdx::newBackRef(/*largeObj=*/true);
     setBackRef(headerLO->backRefIdx, headerLO);
-    ASSERT(scalable_msize(falseLO) == blockSize + headersSize,
+    ASSERT(scalable_msize(falseLO) == slabSize + headersSize,
            "Error in test: LOH falsification failed");
     removeBackRef(headerLO->backRefIdx);
 
@@ -432,30 +572,29 @@ void TestObjectRecognition() {
     scalable_free(smallPtr);
 
     obtainedSize = safer_scalable_msize(mem, NULL);
-    ASSERT(obtainedSize>=2*blockSize, "Correct pointer not accepted?");
+    ASSERT(obtainedSize>=2*slabSize, "Correct pointer not accepted?");
     scalable_free(mem);
     scalable_free(bufferLOH);
 }
 
-class TestBackendWork: NoAssign {
+class TestBackendWork: public SimpleBarrier {
     struct TestBlock {
         intptr_t   data;
         BackRefIdx idx;
     };
-    static const int ITERS = 100;
+    static const int ITERS = 20;
 
-    Harness::SpinBarrier *barrier;
     rml::internal::Backend *backend;
 public:
-    TestBackendWork(rml::internal::Backend *backend, Harness::SpinBarrier *barrier) : 
-        backend(backend), barrier(barrier) {}
+    TestBackendWork(rml::internal::Backend *bknd) : backend(bknd) {}
     void operator()(int) const {
-        barrier->wait();
-        
+        barrier.wait();
+
         for (int i=0; i<ITERS; i++) {
-            BlockI *block16K = backend->get16KBlock(1, /*startup=*/false);
-            LargeMemoryBlock *lmb = backend->getLargeBlock(16*1024, /*startup=*/false);
-            backend->put16KBlock(block16K, /*startup=*/false);
+            BlockI *slabBlock = backend->getSlabBlock(1);
+            ASSERT(slabBlock, "Memory was not allocated");
+            LargeMemoryBlock *lmb = backend->getLargeBlock(8*1024);
+            backend->putSlabBlock(slabBlock);
             backend->putLargeBlock(lmb);
         }
     }
@@ -463,31 +602,70 @@ public:
 
 void TestBackend()
 {
-    rml::MemPoolPolicy pol;
-    Harness::SpinBarrier barrier;
-
-    pol.pAlloc = getMallocMem;
-    pol.pFree = putMallocMem;
-    pol.granularity = 8;
-    rml::MemoryPool *mPool = pool_create(0, &pol);
-    rml::internal::ExtMemoryPool *ePool = 
+    rml::MemPoolPolicy pol(getMallocMem, putMallocMem);
+    rml::MemoryPool *mPool;
+    pool_create_v1(0, &pol, &mPool);
+    rml::internal::ExtMemoryPool *ePool =
         &((rml::internal::MemoryPool*)mPool)->extMemPool;
     rml::internal::Backend *backend = &ePool->backend;
 
     for( int p=MaxThread; p>=MinThread; --p ) {
-        barrier.initialize(p);
-        NativeParallelFor( p, TestBackendWork(backend, &barrier) );
+        // regression test against an race condition in backend synchronization,
+        // triggered only when WhiteboxTestingYield() call yields
+        for (int i=0; i<100; i++) {
+            TestBackendWork::initBarrier(p);
+            NativeParallelFor( p, TestBackendWork(backend) );
+        }
     }
 
-    BlockI *block = backend->get16KBlock(1, /*startup=*/false);
-    backend->put16KBlock(block, /*startup=*/false);
+    BlockI *block = backend->getSlabBlock(1);
+    ASSERT(block, "Memory was not allocated");
+    backend->putSlabBlock(block);
 
     pool_destroy(mPool);
 }
 
+void TestBitMask()
+{
+    BitMaskMin<256> mask;
+
+    mask.reset();
+    mask.set(10, 1);
+    mask.set(5, 1);
+    mask.set(1, 1);
+    ASSERT(mask.getMinTrue(2) == 5, NULL);
+
+    mask.reset();
+    mask.set(0, 1);
+    mask.set(64, 1);
+    mask.set(63, 1);
+    mask.set(200, 1);
+    mask.set(255, 1);
+    ASSERT(mask.getMinTrue(0) == 0, NULL);
+    ASSERT(mask.getMinTrue(1) == 63, NULL);
+    ASSERT(mask.getMinTrue(63) == 63, NULL);
+    ASSERT(mask.getMinTrue(64) == 64, NULL);
+    ASSERT(mask.getMinTrue(101) == 200, NULL);
+    ASSERT(mask.getMinTrue(201) == 255, NULL);
+    mask.set(255, 0);
+    ASSERT(mask.getMinTrue(201) == -1, NULL);
+}
+
+void checkNoHugePages()
+{
+    ASSERT(!hugePages.enabled, "scalable_allocation_mode "
+           "must have priority over environment variable");
+}
+
 int TestMain () {
+    scalable_allocation_mode(USE_HUGE_PAGES, 0);
+#if !_XBOX && !__TBB_WIN8UI_SUPPORT
+    putenv((char*)"TBB_MALLOC_USE_HUGE_PAGES=yes");
+#endif
+    checkNoHugePages();
     // backreference requires that initialization was done
     if(!isMallocInitialized()) doInitialization();
+    checkNoHugePages();
     // to succeed, leak detection must be the 1st memory-intensive test
     TestBackRef();
     TestPools();
@@ -501,15 +679,8 @@ int TestMain () {
     }
 #endif
 
-    for (int i=0; i<LARGE_MEM_SIZES_NUM; i++)
-        TestLargeObjCache::largeMemSizes[i] = 
-            (int)(minLargeObjectSize + 2*minLargeObjectSize*(1.*rand()/RAND_MAX));
-
-    for( int p=MaxThread; p>=MinThread; --p ) {
-        TestLargeObjCache::initBarrier( p );
-        NativeParallelFor( p, TestLargeObjCache() );
-    }
-
+    TestLargeObjectCache();
     TestObjectRecognition();
+    TestBitMask();
     return Harness::Done;
 }
