@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -189,7 +189,7 @@ struct parallel_put_get : NoAssign {
 // Item can be reserved, released, consumed ( single serial receiver )
 //
 template< typename T >
-int test_reservation(int num_threads) {
+int test_reservation() {
     tbb::flow::graph g;
     T bogus_value(-1);
 
