@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -30,14 +30,15 @@
 #include "tbb/reader_writer_lock.h"
 #include "tbb/atomic.h"
 #include "tbb/tbb_exception.h"
-#include "harness_assert.h"
 #include "harness.h"
+#include "harness_barrier.h"
 
 tbb::reader_writer_lock the_mutex;
 const int MAX_WORK = 10000;
 
 tbb::atomic<size_t> active_readers, active_writers;
 tbb::atomic<bool> sim_readers;
+size_t n_tested__sim_readers;
 
 
 int BusyWork(int percentOfMaxWork) {
@@ -91,7 +92,7 @@ struct StressRWLBody : NoAssign {
 #endif /* TBB_USE_EXCEPTIONS && !__TBB_THROW_ACROSS_MODULE_BOUNDARY_BROKEN */
             the_mutex.unlock();
             // test unscoped non-blocking write lock
-            if (the_mutex.try_lock()) { 
+            if (the_mutex.try_lock()) {
                 w_result += BusyWork(percentMax);
                 the_mutex.unlock();
             }
@@ -100,7 +101,7 @@ struct StressRWLBody : NoAssign {
             r_result += BusyWork(percentMax);
             the_mutex.unlock();
             // test unscoped non-blocking read lock
-            if(the_mutex.try_lock_read()) { 
+            if(the_mutex.try_lock_read()) {
                 r_result += BusyWork(percentMax);
                 the_mutex.unlock();
             }
@@ -113,21 +114,20 @@ struct StressRWLBody : NoAssign {
                 r_result += BusyWork(percentMax);
             }
         }
-        REMARK("%d reader %d writer iterations of busy work were completed.", r_result, w_result);
+        REMARK(" R%d/W%d", r_result, w_result); // reader/writer iterations of busy work completed
     }
 };
 
 struct CorrectRWLScopedBody : NoAssign {
     const int nThread;
+    Harness::SpinBarrier& my_barrier;
 
-    CorrectRWLScopedBody(int nThread_) : nThread(nThread_) {}
+    CorrectRWLScopedBody(int nThread_, Harness::SpinBarrier& b_) : nThread(nThread_),my_barrier(b_) {}
 
     void operator()(const int /* threadID */ ) const {
-        bool is_reader;
-        
+        my_barrier.wait();
         for (int i=0; i<50; i++) {
-            if (i%5==0) is_reader = false; // 1 writer for every 5 readers
-            else is_reader = true;
+            const bool is_reader = i%5==0; // 1 writer for every 4 readers
 
             if (is_reader) {
                 tbb::reader_writer_lock::scoped_lock_read my_lock(the_mutex);
@@ -135,7 +135,7 @@ struct CorrectRWLScopedBody : NoAssign {
                 if (active_readers > 1) sim_readers = true;
                 ASSERT(active_writers==0, "Active writers in read-locked region.");
                 Harness::Sleep(10);
-                active_readers--; 
+                active_readers--;
             }
             else { // is writer
                 tbb::reader_writer_lock::scoped_lock my_lock(the_mutex);
@@ -151,15 +151,14 @@ struct CorrectRWLScopedBody : NoAssign {
 
 struct CorrectRWLBody : NoAssign {
     const int nThread;
+    Harness::SpinBarrier& my_barrier;
 
-    CorrectRWLBody(int nThread_) : nThread(nThread_) {}
+    CorrectRWLBody(int nThread_, Harness::SpinBarrier& b_ ) : nThread(nThread_), my_barrier(b_) {}
 
     void operator()(const int /* threadID */ ) const {
-        bool is_reader;
-        
+        my_barrier.wait();
         for (int i=0; i<50; i++) {
-            if (i%5==0) is_reader = false; // 1 writer for every 5 readers
-            else is_reader = true;
+            const bool is_reader = i%5==0; // 1 writer for every 4 readers
 
             if (is_reader) {
                 the_mutex.lock_read();
@@ -175,7 +174,7 @@ struct CorrectRWLBody : NoAssign {
             }
             Harness::Sleep(10);
             if (is_reader) {
-                active_readers--; 
+                active_readers--;
             }
             else { // is writer
                 active_writers--;
@@ -187,27 +186,48 @@ struct CorrectRWLBody : NoAssign {
 
 void TestReaderWriterLockOnNThreads(int nThreads) {
     // Stress-test all interfaces
-    for (int pc=0; pc<101; pc+=20) {
-        REMARK("\nTesting reader_writer_lock with %d threads, percent of MAX_WORK=%d", nThreads, pc);
+    for (int pc=0; pc<=100; pc+=20) {
+        REMARK("Testing with %d threads, percent of MAX_WORK=%d...", nThreads, pc);
         StressRWLBody myStressBody(nThreads, pc);
         NativeParallelFor(nThreads, myStressBody);
+        REMARK(" OK.\n");
     }
-    
-    // Test mutual exclusion in direct locking mode
-    CorrectRWLBody myCorrectBody(nThreads);
-    active_writers = active_readers = 0;
-    sim_readers = false;
-    NativeParallelFor(nThreads, myCorrectBody);
-    ASSERT(sim_readers || nThreads<2, "There were no simultaneous readers.");
-    REMARK("Unscoped lock testing succeeded on %d threads.", nThreads);
 
-    // Test mutual exclusionin scoped locking mode
-    CorrectRWLScopedBody myCorrectScopedBody(nThreads);
-    active_writers = active_readers = 0;
-    sim_readers = false;
-    NativeParallelFor(nThreads, myCorrectScopedBody);
-    ASSERT(sim_readers || nThreads<2, "There were no simultaneous readers.");
-    REMARK("Scoped lock testing succeeded on %d threads.", nThreads);
+    int i;
+    n_tested__sim_readers = 0;
+    REMARK("Testing with %d threads, direct/unscoped locking mode...", nThreads); // TODO: choose direct or unscoped?
+    // TODO: refactor the following two for loops into a shared function 
+    for( i=0; i<100; ++i ) {
+        Harness::SpinBarrier bar0(nThreads);
+
+        CorrectRWLBody myCorrectBody(nThreads,bar0);
+        active_writers = active_readers = 0;
+        sim_readers = false;
+        NativeParallelFor(nThreads, myCorrectBody);
+
+        if( sim_readers || nThreads==1 ) {
+            if( ++n_tested__sim_readers>5 )
+                break;
+        }
+    }
+    ASSERT(i<100, "There were no simultaneous readers.");
+    REMARK(" OK.\n");
+
+    n_tested__sim_readers = 0;
+    REMARK("Testing with %d threads, scoped locking mode...", nThreads);
+    for( i=0; i<100; ++i ) {
+        Harness::SpinBarrier bar0(nThreads);
+        CorrectRWLScopedBody myCorrectScopedBody(nThreads, bar0);
+        active_writers = active_readers = 0;
+        sim_readers = false;
+        NativeParallelFor(nThreads, myCorrectScopedBody);
+        if( sim_readers || nThreads==1 ) {
+            if( ++n_tested__sim_readers>5 )
+                break;
+        }
+    }
+    ASSERT(i<100, "There were no simultaneous readers.");
+    REMARK(" OK.\n");
 }
 
 void TestReaderWriterLock() {
