@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -26,8 +26,12 @@
     the GNU General Public License.
 */
 
+#if __TBB_CPF_BUILD
+    #define TBB_PREVIEW_TASK_ARENA 1
+#endif
 #include "harness.h"
 
+
 #if __TBB_TASK_GROUP_CONTEXT
 
 #include "tbb/task.h"
@@ -35,10 +39,15 @@
 #include "tbb/atomic.h"
 #include <cstdlib>
 
+#if _MSC_VER && __TBB_NO_IMPLICIT_LINKAGE
+// plays around __TBB_NO_IMPLICIT_LINKAGE. __TBB_LIB_NAME should be defined (in makefiles)
+    #pragma comment(lib, __TBB_STRING(__TBB_LIB_NAME))
+#endif
+
 const int NumIterations = 100;
 const int NumLeafTasks = 2;
-int MinBaseDepth = 9;
-int MaxBaseDepth = 11;
+int MinBaseDepth = 8;
+int MaxBaseDepth = 10;
 int BaseDepth = 0;
 
 const int NumTests = 8;
@@ -73,6 +82,13 @@ enum Options {
     FlogEncloser = Flog | 4
 };
 
+const char *PriorityName(tbb::priority_t p) {
+    if( p == tbb::priority_high ) return "high";
+    if( p == tbb::priority_normal ) return "normal";
+    if( p == tbb::priority_low ) return "low";
+    return "bad";
+}
+
 void PrepareGlobals ( int numMasters ) {
     ASSERT( !g_NumMasters && !g_LeavesExecuted, NULL );
     g_NumMasters = numMasters;
@@ -228,7 +244,7 @@ public:
                 tbb::task::enqueue( *new(tbb::task::allocate_root() ) tbb::empty_task, id == 0 ? Low : High );
                 Harness::Sleep(count);
 #if __TBB_ipf
-                // Increased sleep periods are required on systems with unfair hyperthreading (Itanium 2)
+                // Increased sleep periods are required on systems with unfair hyperthreading (Itanium(R) 2 processor)
                 count += 10;
 #endif
             }
@@ -277,7 +293,7 @@ public:
 template<class NodeType>
 void RunPrioritySwitchBetweenTwoMasters ( int idx, uintptr_t opts ) {
     ASSERT( idx < NumTests, NULL );
-    REMARK( "Config %d\r", ++g_CurConfig );
+    REMARK( "Config %d: idx=%i, opts=%u\r", ++g_CurConfig, idx, (unsigned)opts );
     NativeParallelFor ( 2, MasterBody<NodeType>(idx, opts) );
     Harness::Sleep(50);
 }
@@ -388,18 +404,74 @@ void TestPriorityAssertions () {
     tbb::task &t = *new( tbb::task::allocate_root() ) tbb::empty_task;
     TRY_BAD_EXPR( tbb::task::enqueue( t, bad_high_priority ), "Invalid priority level value" );
     // Restore normal assertion handling
-    tbb::set_assertion_handler( NULL );
+    tbb::set_assertion_handler( ReportError );
 #endif /* TRY_BAD_EXPR_ENABLED && __TBB_TASK_PRIORITY */
 }
 
+#if __TBB_TASK_PRIORITY
+tbb::atomic<tbb::priority_t> g_order;
+tbb::atomic<bool> g_order_established;
+class OrderedTask : public tbb::task {
+    tbb::priority_t my_priority;
+public:
+    OrderedTask(tbb::priority_t p) : my_priority(p) {}
+    tbb::task* execute() {
+        tbb::priority_t prev = g_order.fetch_and_store(my_priority);
+        if( my_priority != prev) {
+            REMARK("prev:%s --> new:%s\n", PriorityName(prev), PriorityName(my_priority));
+            // TODO: improve the test for concurrent workers
+            if(!g_order_established) {
+                // initial transition path allowed low->[normal]->high
+                if(my_priority == tbb::priority_high)
+                    g_order_established = true;
+                else ASSERT(my_priority == tbb::priority_normal && prev == tbb::priority_low, NULL);
+            } else { //transition path allowed high->normal->low
+                if(prev == tbb::priority_high) ASSERT( my_priority == tbb::priority_normal, "previous priority is high - bad order");
+                else if(prev == tbb::priority_normal) ASSERT( my_priority == tbb::priority_low, "previous priority is normal - bad order");
+                else ASSERT(!g_order_established, "transition from low priority but not during initialization");
+            }
+        }
+        EmulateWork(0);
+        return NULL;
+    }
+    static void start(int i) {
+        tbb::priority_t p = i%3==0? tbb::priority_low : (i%3==1? tbb::priority_normal : tbb::priority_high );
+        OrderedTask &t = *new(tbb::task::allocate_root()) OrderedTask(p);
+        tbb::task::enqueue(t, p);
+    }
+};
+
+//Look for discussion of the issue at http://software.intel.com/en-us/forums/showthread.php?t=102159
+void TestEnqueueOrder () {
+    REMARK("Testing order of enqueued tasks\n");
+    tbb::task_scheduler_init init(1); // to simplify transition checks only one extra worker for enqueue
+    g_order = tbb::priority_low;
+    g_order_established = false;
+    for( int i = 0; i < 1000; i++)
+        OrderedTask::start(i);
+    while( g_order == tbb::priority_low ) __TBB_Yield();
+    while( g_order != tbb::priority_low ) __TBB_Yield();
+}
+#endif /* __TBB_TASK_PRIORITY */
+
+#if !__TBB_TEST_SKIP_AFFINITY
+#include "harness_concurrency.h"
+#endif
+
 int TestMain () {
+#if !__TBB_TEST_SKIP_AFFINITY
+    Harness::LimitNumberOfThreads( 16 );
+#endif
 #if !__TBB_TASK_PRIORITY
     REMARK( "Priorities disabled: Running as just yet another task scheduler test\n" );
+#else
+    TestEnqueueOrder();
 #endif /* __TBB_TASK_PRIORITY */
     TestPriorityAssertions();
     TestSimplePriorityOps(tbb::priority_low);
     TestSimplePriorityOps(tbb::priority_high);
     P = tbb::task_scheduler_init::default_num_threads();
+    REMARK( "The number of threads: %d\n", P );
     if ( P < 3 )
         return Harness::Skipped;
     TestPeriodicConcurrentActivities();
