@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -28,6 +28,15 @@
 
 #include "tbb/tbb_stddef.h"
 
+#if __TBB_WIN8UI_SUPPORT
+// TODO: figure out how the test can be enabled for win8ui
+#define HARNESS_NO_PARSE_COMMAND_LINE 1
+#include "harness.h"
+int TestMain() {
+    return Harness::Skipped;
+}
+#else
+
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -61,7 +70,8 @@
 
 enum string_required {
     required,
-    not_required
+    optional,
+    optional_multiple
     };
 
 typedef std::pair <std::string, string_required> string_pair;
@@ -73,22 +83,38 @@ const char stdout_stream[] = "version_test.out";
 
 HARNESS_EXPORT
 int main(int argc, char *argv[] ) {
+    const size_t psBuffer_len = 512;
+    char psBuffer[psBuffer_len];
 /* We first introduced runtime version identification in 3014 */
-#if TBB_INTERFACE_VERSION>=3014 
+#if TBB_INTERFACE_VERSION>=3014
     // For now, just test that run-time TBB version matches the compile-time version,
     // since otherwise the subsequent test of "TBB: INTERFACE VERSION" string will fail anyway.
     // We need something more clever in future.
-    ASSERT(tbb::TBB_runtime_interface_version()==TBB_INTERFACE_VERSION,
-           "Running with the library of different version than the test was compiled against");
+    if ( tbb::TBB_runtime_interface_version()!=TBB_INTERFACE_VERSION ){
+        snprintf( psBuffer, psBuffer_len,
+                  "%s %s %d %s %d.",
+                  "Running with the library of different version than the test was compiled against.",
+                  "Expected",
+                  TBB_INTERFACE_VERSION,
+                  "- got",
+                  tbb::TBB_runtime_interface_version()
+                  );
+        ASSERT( tbb::TBB_runtime_interface_version()==TBB_INTERFACE_VERSION, psBuffer );
+    }
 #endif
+#if __TBB_MIC_OFFLOAD
+    // Skip the test in offload mode.
+    // Run the test in 'true' native mode (because 'system()' works in 'true' native mode).
+    (argc, argv);
+    REPORT("skip\n");
+#else //__TBB_MIC_OFFLOAD
 #if __TBB_MPI_INTEROP
     REPORT("skip\n");
 #else
     __TBB_TRY {
         FILE *stream_out;
-        FILE *stream_err;   
-        char psBuffer[512];
-        
+        FILE *stream_err;
+
         if(argc>1 && argv[1][0] == '@' ) {
             stream_err = freopen( stderr_stream, "w", stderr );
             if( stream_err == NULL ){
@@ -122,7 +148,7 @@ int main(int argc, char *argv[] ) {
                 exit( 1 );
             }
             while( !feof( stream_err ) ) {
-                if( fgets( psBuffer, 512, stream_err ) != NULL ){
+                if( fgets( psBuffer, psBuffer_len, stream_err ) != NULL ){
                     REPORT( "Error (step 1): stderr should be empty\n" );
                     exit( 1 );
                 }
@@ -134,7 +160,7 @@ int main(int argc, char *argv[] ) {
                 exit( 1 );
             }
             while( !feof( stream_out ) ) {
-                if( fgets( psBuffer, 512, stream_out ) != NULL ){
+                if( fgets( psBuffer, psBuffer_len, stream_out ) != NULL ){
                     REPORT( "Error (step 1): stdout should be empty\n" );
                     exit( 1 );
                 }
@@ -164,7 +190,7 @@ int main(int argc, char *argv[] ) {
             exit( 1 );
         }
         while( !feof( stream_out ) ) {
-            if( fgets( psBuffer, 512, stream_out ) != NULL ){
+            if( fgets( psBuffer, psBuffer_len, stream_out ) != NULL ){
                 REPORT( "Error (step 2): stdout should be empty\n" );
                 exit( 1 );
             }
@@ -177,30 +203,32 @@ int main(int argc, char *argv[] ) {
             exit( 1 );
         }
         
-        int skip_line = 0;
-        
         while( !feof( stream_err ) ) {
-            if( fgets( psBuffer, 512, stream_err ) != NULL ){
+            if( fgets( psBuffer, psBuffer_len, stream_err ) != NULL ){
+                if (strstr( psBuffer, "TBBmalloc: " )) {
+                    // TBB allocator might or might not be here, ignore it
+                    continue;
+                }
+                bool match_found = false;
                 do{
                     if ( strings_iterator == strings_vector.end() ){
                         REPORT( "Error: version string dictionary ended prematurely.\n" );
                         REPORT( "No match for: \t%s", psBuffer );
                         exit( 1 );
                     }
-                    if ( strstr( psBuffer, strings_iterator->first.c_str() ) == NULL ){
+                    if ( strstr( psBuffer, strings_iterator->first.c_str() ) == NULL ){ // mismatch
                         if( strings_iterator->second == required ){
                             REPORT( "Error: version strings do not match.\n" );
                             REPORT( "Expected \"%s\" not found in:\n\t%s", strings_iterator->first.c_str(), psBuffer );
                             exit( 1 );
-                        }else{
-                            //Do we need to print in case there is no non-required string?
-                            skip_line = 1;
                         }
+                        ++strings_iterator;
                     }else{
-                           skip_line = 0;
+                        match_found = true;
+                        if( strings_iterator->second != optional_multiple )
+                            ++strings_iterator;
                     }
-                    if ( strings_iterator != strings_vector.end() ) strings_iterator ++;
-                }while( skip_line );
+                }while( !match_found );
             }
         }
         fclose( stream_err );
@@ -209,6 +237,7 @@ int main(int argc, char *argv[] ) {
     }
     REPORT("done\n");
 #endif //__TBB_MPI_INTEROP
+#endif //__TBB_MIC_OFFLOAD
     return 0;
 }
 
@@ -216,8 +245,8 @@ int main(int argc, char *argv[] ) {
 // Fill dictionary with version strings for platforms 
 void initialize_strings_vector(std::vector <string_pair>* vector)
 {
-    vector->push_back(string_pair("TBB: VERSION\t\t4.0", required));          // check TBB_VERSION
-    vector->push_back(string_pair("TBB: INTERFACE VERSION\t6000", required)); // check TBB_INTERFACE_VERSION
+    vector->push_back(string_pair("TBB: VERSION\t\t4.1", required));          // check TBB_VERSION
+    vector->push_back(string_pair("TBB: INTERFACE VERSION\t6105", required)); // check TBB_INTERFACE_VERSION
     vector->push_back(string_pair("TBB: BUILD_DATE", required));
     vector->push_back(string_pair("TBB: BUILD_HOST", required));
     vector->push_back(string_pair("TBB: BUILD_OS", required));
@@ -229,18 +258,27 @@ void initialize_strings_vector(std::vector <string_pair>* vector)
 #elif __APPLE__
     vector->push_back(string_pair("TBB: BUILD_KERNEL", required));
     vector->push_back(string_pair("TBB: BUILD_GCC", required));
-    vector->push_back(string_pair("TBB: BUILD_COMPILER", not_required)); //if( getenv("COMPILER_VERSION") )
+    vector->push_back(string_pair("TBB: BUILD_COMPILER", optional)); //if( getenv("COMPILER_VERSION") )
 #elif __sun
     vector->push_back(string_pair("TBB: BUILD_KERNEL", required));
     vector->push_back(string_pair("TBB: BUILD_SUNCC", required));
-    vector->push_back(string_pair("TBB: BUILD_COMPILER", not_required)); //if( getenv("COMPILER_VERSION") )
-#else //We use version_info_linux.sh for unsupported OSes
+    vector->push_back(string_pair("TBB: BUILD_COMPILER", optional)); //if( getenv("COMPILER_VERSION") )
+#else // We use version_info_linux.sh for unsupported OSes
+#if __ANDROID__
+    vector->push_back(string_pair("TBB: BUILD_TARGET_OS", required));
+    vector->push_back(string_pair("TBB: BUILD_TARGET_KERNEL", required));
+#else
     vector->push_back(string_pair("TBB: BUILD_KERNEL", required));
+#endif // !__ANDROID__
     vector->push_back(string_pair("TBB: BUILD_GCC", required));
-    vector->push_back(string_pair("TBB: BUILD_COMPILER", not_required)); //if( getenv("COMPILER_VERSION") )
-    vector->push_back(string_pair("TBB: BUILD_GLIBC", required));
+    vector->push_back(string_pair("TBB: BUILD_COMPILER", optional)); //if( getenv("COMPILER_VERSION") )
+#if __ANDROID__
+    vector->push_back(string_pair("TBB: BUILD_NDK", optional));
+#else
+    vector->push_back(string_pair("TBB: BUILD_LIBC", required));
+#endif // !__ANDROID__
     vector->push_back(string_pair("TBB: BUILD_LD", required));
-#endif
+#endif // OS
     vector->push_back(string_pair("TBB: BUILD_TARGET", required));
     vector->push_back(string_pair("TBB: BUILD_COMMAND", required));
     vector->push_back(string_pair("TBB: TBB_USE_DEBUG", required));
@@ -249,11 +287,16 @@ void initialize_strings_vector(std::vector <string_pair>* vector)
     vector->push_back(string_pair("TBB: TBB_PREVIEW_BINARY", required));
 #endif
     vector->push_back(string_pair("TBB: DO_ITT_NOTIFY", required));
-    vector->push_back(string_pair("TBB: ITT", not_required)); //#ifdef DO_ITT_NOTIFY
+    vector->push_back(string_pair("TBB: ITT", optional)); //#ifdef DO_ITT_NOTIFY
     vector->push_back(string_pair("TBB: ALLOCATOR", required));
-    vector->push_back(string_pair("TBB: RML", not_required));
-    vector->push_back(string_pair("TBB: Intel(R) RML library built:", not_required));
-    vector->push_back(string_pair("TBB: Intel(R) RML library version:", not_required));
+#if _WIN32||_WIN64
+    vector->push_back(string_pair("TBB: Processor groups", required));
+    vector->push_back(string_pair("TBB: ----- Group", optional_multiple));
+#endif
+    vector->push_back(string_pair("TBB: RML", optional));
+    vector->push_back(string_pair("TBB: Intel(R) RML library built:", optional));
+    vector->push_back(string_pair("TBB: Intel(R) RML library version:", optional));
     vector->push_back(string_pair("TBB: Tools support", required));
     return;
 }
+#endif /* __TBB_WIN8UI_SUPPORT */
